function [Mati,Mad] = findMatiMadBoundry(ncs,plotFlag)
%FINDMATIMADBOUNDRY  Verifies LMI conditions on a hybrid NCS model to generate two vectors 
%                    used for plotting a robust stability region corresponing to an NCS.
%                    Model and analysis conditions are based on:
%                    [Heemels et. al., Trans. Autom. Control 2010]
%
%   [MATI,MAD] = FINDMATIMADBOUNDRY(NCS) Generates the vectors MATI, MAD needed to plot 
%   a maximum allowable transmission interval (MATI), maxmium allowable delay (MAD)
%   tradeoff curve which defines the region where robust stability can be guaranteed
%   for an NCS.  
%
%   INPUTS
%   ------------------------------------------------------
%      NCS   - NCS object created with the ncsEditor
%
%   OUTPUTS
%   ------------------------------------------------------
%      MATI - vector of values for MATI,MAD tradeoff plot
%      MAD -  vector of values for MATI,MAD tradeoff plot
%
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% Generate Hybrid Model
    hbMdl = genHybNcsModel(ncs);

    % test closed loop stability without network
    if sum(eig(hbMdl.A11) >= 0) > 0
        error('Closed loop system without network is unstable: reconsider controller; if Ac = 0, use a small negative constant or use the static controller function')
    end
    
    %% Generate MATI MAD vectors to plot tradeoff curve
    
    l=length(ncs.getNetwork.nodesU);
    [Mati,Mad] = analyzeHybNcs(hbMdl,l,ncs.getNetwork.protocol);
    
    if nargin==1 || plotFlag==1
        figure;
        plot(Mati,Mad);
        xlabel('Maximum Allowable Transmission Interval (s)')
        ylabel('Maximum Allowable Delay (s)')
        M = max([Mati; Mad]);
        hold all;
        plot([0 M], [0 M], 'k--')
        legend([ncs.getNetwork.protocol ' Stability Boundry'],'Small-Delay Boundry')
    end
    fprintf('-------------------------------------------------------------- \n')