function obj = genTPoly(obj,delta,res,plotFlag)
%GENTPOLY  Determines the polygon in the (txIntvl,delay) uncertainty space 
%          based on the timing distribution in the NCS object.
%
%   NCS = GENTPOLY(NCS,DELTA,RES,PLOTFLAG) Determines a region in the
%   (txIntvl,delay) uncertainty space based on the timing distribution in
%   the NCS object, the DELTA percent of most-likely occuring points and a
%   resolution RES around each point.
%
%   INPUTS
%   ------------------------------------------------------
%      NCS      - NCS object
%      DELTA    - (0,1] threshold which specifies to create a polygon which
%                  contain the DELTA percent of most-likely occurring
%                  points
%      RES      - a decimal which specifies the measurement resolution
%      PLOTFLAG - {0,1} specifies whether to plot the uncertain polygon
%                  along with the data it was based on
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if nargin==3
       plotFlag=0; 
    end
    
    if isempty(obj.timingDist)
        error('Timing distribution must be specified before genTPoly can be run');
    end
    
    pts_sort = obj.timingDist;
    %------------------------------------------------------------
    % Eliminate points based on threshold
    
    highestProb = pts_sort(1,3);
    if all(pts_sort(:,3)<delta)
        obj.setNetwork('tPoly',[]);
        return;
    else
        cutoff = (1-delta)*highestProb;
    end

    cutoff_index=find(pts_sort(:,3)>=cutoff,1,'last');
    timingDist = pts_sort(1:cutoff_index,:);    
    
%------------------------------------------------------------
    % Add boxing to account for resolution
    sqrs = cell(size(timingDist,1),1);
    color = zeros(size(timingDist,1),1);
    
    for i = 1: size(timingDist,1)
        pt1 = [timingDist(i,1)+0.5*res timingDist(i,2)+0.5*res];
        pt2 = [timingDist(i,1)-0.5*res timingDist(i,2)+0.5*res];
        pt3 = [timingDist(i,1)-0.5*res timingDist(i,2)-0.5*res];
        pt4 = [timingDist(i,1)+0.5*res timingDist(i,2)-0.5*res];
        color(i) = timingDist(i,3);
        sqrs{i} = [pt1; pt2; pt3; pt4]; 
    end
    
    
    % only take vertices of convex hull
    sel =vertcat(sqrs{:});
    sel_hk = sel(:,1);
    sel_tau = sel(:,2);
    sel_indx = convhull(sel_hk,sel_tau,'simplify',true);
    tPoly = [sel_hk(sel_indx),sel_tau(sel_indx)];
    tPoly=tPoly(1:end-1,:); %last point is same as first, so remove it
    
    obj.tPoly=tPoly;
    
    if plotFlag
        figure;
        colormap autumn;
        colorbar;
        patch(tPoly(:,1),tPoly(:,2),'r');
       % set colorbar, scaled to repetitions
        set(gca,'CLim',[pts_sort(end,3) pts_sort(1,3)]);
         for i=1:length(sqrs)
            dataPlot(i) = patch(sqrs{i}(:,1),sqrs{i}(:,2),'r');
            set(dataPlot(i),'FaceColor','flat','FaceVertexCData',color(i),'CDataMapping','scaled');
         end  
         
    end
    
    