function controller = getController(object)
% GETCONTROLLER Returns the controller of an NCS object. 
%
% NCS = GETCONTROLLER(NCS) returns the CTLTYPE of controller along with the
% corresponding system matrices.
%
% CTLTYPE = 'Static Feedback' corresponds to the controller
%
%         u = K{ell}*\hat{y}
%
% where ell correpsonds to the node which receives network access.
%
% CTLTYPE = 'C-LTI Dynamic Feedback' corresponds to the controller
%
%   \dot{z} = A{ell}*z + B{ell}*\hat{y}
%         u = C{ell}*\hat{y} + D{ell}*\hat{u}
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% CTLTYPE = 'D-LTI Dynamic Feedback' correpsonds to the controller
%
%   z_{k+1} = A{ell}*z_k + B{ell}*hat{y}_k
%       u_k = C{ell}*z_k + D{ell}*\hat{u}_k
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% CTLTYPE = 'D-LTI Dynamic Observer-Based Feedback' Corresponds to the
% controller
%
%   z_{k+1} = A{ell}*z_k + B{ell}*hat{u}_k - L{ell}*(\hat{y}_k-C{ell}*z_k)
%       u_k = K{ell}*z_k
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% See also SETCONTROLLER
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    if strcmp(object.ctrlType,'Static Feedback')
        controller.ctrlType=object.ctrlType;
        controller.K=object.K;
    elseif strcmp(object.ctrlType,'C-LTI Dynamic Feedback') || ...
            strcmp(object.ctrlType,'D-LTI Dynamic Feedback' ) || ...
             strcmp(object.ctrlType,'C-LTI Dynamic Feedback (uWired)' )
        controller.ctrlType=object.ctrlType;
        controller.A=object.Ac;
        controller.B=object.Bc;
        controller.C=object.Cc;
        controller.D=object.Dc;
    elseif strcmp(object.ctrlType,'D-LTI Observer-Based Feedback')
        controller.ctrlType=object.ctrlType;
        controller.A=object.Ac;
        controller.B=object.Bc;
        controller.C=object.Cc;
        controller.K=object.K;
        controller.L=object.L;
    else
        error('Controller type unknown')
    end
    
    %SYS = ss(A,B,C,zeros(size(B,2),size(C,1)));
end