function ovrAprx = getPolyOvrAprx(obj,ovraprx,arg1,arg2)
%GETPOLYOVRAPRX  from a NCS object, this functions generates a polytopic 
%                overapproximation of the form 
%
%    x_{k+1} = (\sum_{m=1}^{M}alpha^m_k* Ab{l,m} + Bb{l}* Delta_k *Cb{l})x_{k} +
%               (\sum_{m=1}^{M}alpha^m_k* Eb{l,m} + Db{l}* Delta_k *Jb{l})w_{k}
%
%               where x_k is the value of the NCS state vector at time k, w_k 
%               is a disturbance and Delta_k belongs to a set of structured 
%               norm bounded matrices. See [Bauer et. al. NECSYS, 2012] eqn 
%               (2) for more info. Overapproximation procedures are based on:
% 
%                   GNB - [Donkers et. al., Trans. Autom. Control 2011]
%                   JNF - [Cloosterman et. al., Trans. Autom. Control 2009]
%                   CH  - [Gielen et. al., Automatica 2010]
%
%   POLYOVRAPRX = GETPOLYOVRAPRX(NCS,'JNF') 
%   Generates the polytopic overapproximation POLYOVRAPRX using the 
%   overapprimation JNF.
%
%   POLYOVRAPRX = GETPOLYOVRAPRX(NCS,'CH') 
%   Generates the polytopic overapproximation POLYOVRAPRX using the 
%   the overapprimation CH.
%
%   POLYOVRAPRX = GETPOLYOVRAPRX(NCS,'GNB',EPS,GPT_MAX) 
%   Generates the polytopic overapproximation POLYOVRAPRX using the 
%   overapprimation GNB.  EPS is a floating-point number indicating
%   the overapproximation tightness (lower value = tighter overaproximation) 
%   and GPT_MAX is an integer indicating the maximum gridpoints allowable.
%
%   OUTPUT
%   ------------------------------------------------------
%      POLYOVRAPRX   - strucuture containing information about
%                      the overapproximated model
%                       ~~~~Fields~~~~~~~~~~~~
%                       POLYOVRAPRX.type        - 'GNB', 'JNF' or 'CH'                       
%                       POLYOVRAPRX.Ab          - Ab{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Bb          - Bb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Cb          - Cb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Db          - Db{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Eb          - Eb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Jb          - Jb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.JordanBlks  - Jordan blocks used for overapproximation
%                       POLYOVRAPRX.alphas      - (JNF,CH) list of symbolic alpha terms 
%                       POLYOVRAPRX.alphaMinMax - (JNF,CH) min and max of alphas used 
%                                                 to create the overapproximation
%                       POLYOVRAPRX.gridpts     - (GNB) list of grid points in (h,tau) space                       
%                       POLYOVRAPRX.trigangles  - (GNB) list of triangles in (h,tau) space
%                       POLYOVRAPRX.deltas      - (GNB) overapproximation deltas per 
%                                                 triangle per jordan block 
%                       POLYOVRAPRX.MaxEps      - (GNB) maximum value of epsilon 
%                       POLYOVRAPRX.MinEps      - (GNB) minimum value of epsilon 
%
%   Customization Tips:
%      *The structure which is output, POLYOVRAPRX, is suitable for use in 
%       custom performance or stability analysis conditions.
%
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    %arg1 = epsU
    %arg2 = gridPntM

    DtMdl = genDtNcsModel(obj);

    if nargin == 4 && strcmp(ovraprx,'GNB')
        ovrAprx = genPolyOvrAprx(DtMdl,'GNB',arg1,arg2,true);
    elseif nargin == 2 && strcmp(ovraprx,'JNF') || strcmp(ovraprx,'CH')
        ovrAprx = genPolyOvrAprx(DtMdl,ovraprx);
    else
        error('Incorrect number of input arguements.')
    end
end