function verts=getTimingUncertaintyRegion(obj,refineFlag)
% GETTIMINGUNCERTAINTYREGION Returns a list of vertices which define the
%                            uncertain space.
%
% VERTS = GETTIMINGUNCERTAINTYREGION(NCS) returns the list of vertices
% corresponding to the refined region (if specified) or else the list of
% vertices corresponding to the uncertainty region defined by the tx
% interval and delay bounds.
%
% VERTS = GETTIMINGUNCERTAINTYREGION(NCS,REFINEFLAG) returns the list of
% vertices corresponding to the refined region (if REFINEFLAG==1) or
% returns the list of vertices corresponding the the uncertainty region 
% defined by the tx interval and delay bounds (if REFINEFLAG==0).
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if nargin == 1 && ~isempty(obj.tPoly)
        refineFlag=1;
    elseif nargin == 1 && isempty(obj.tPoly)
        refineFlag=0;
    elseif nargin ~=2
        error('Incorrect number of input parameters.')
    end

    if ~isempty(obj.tPoly) && refineFlag==1
        verts=obj.tPoly;
    elseif isempty(obj.tPoly) && refineFlag==1
        error('No refinement specified. Use method refineUncertainTimingRegion to specify a refinement.')
    elseif refineFlag==0
        wiggleRoom = 1e-12;   % numerical precision
        taumin = obj.tau(1);
        taumax = obj.tau(2);
        hmin = obj.h(1);
        hmax = obj.h(2);

        if taumax-taumin < wiggleRoom && hmax-hmin < wiggleRoom
            % both delay and transmission interval are constant
            verts = [hmax,taumax];
        elseif taumax-taumin < wiggleRoom
            % constant delay
            verts = [hmin taumax;
                        hmax taumax];
        elseif hmax-hmin < wiggleRoom
            % constant transmission interval
            verts = [hmax taumin;
                        hmax taumax];
        else
            if hmin >= taumax                       % rectangle shape
            verts = [hmin taumin
                        hmin taumax
                        hmax taumax
                        hmax taumin];
            elseif hmin > taumin && hmax > taumax   %bottom is right, top is right
            verts = [hmin   taumin
                        hmin   hmin
                        taumax taumax
                        hmax   taumax
                        hmax   taumin];
            elseif hmin <= taumin && hmax > taumax  %bottom is left, top is right
            verts = [taumin taumin
                        taumax taumax
                        hmax   taumax
                        hmax   taumin];
            elseif hmin > taumin && hmax <= taumax  %bottom is right, top is left
            verts = [hmin taumin
                        hmin hmin
                        hmax hmax
                        hmax taumin];        
            elseif hmin<=taumin && hmax<=taumax     %bottom is left, top is left
            verts = [taumin taumin
                        hmax   hmax
                        hmax   taumin];
            end
        end
    else
        error('Refinement flag must be 0 or 1.')
    end
end