function [stable,polyOvrAprx]=isNcsStable(ncs,type,quantVars,epsU,gridPtMax)
%ISNCSSTABLE   Verifies LMI conditions on a polytopic overapprximation to 
%             determine if robust stability of a ncs class object can be
%             guaranteed. 
%
%   [STABLE,OVRAPRX]=ISNCSSTABLE(NCS,TYPE,QUANTVARS,EPSU,GRIDPNTMAX)
%   Verifies LMI conditions on a polytopic overapprximation OVRAPRX of an  
%   NCS to determine if robust stability can be guaranteed STABLE.
%
%   This function first runs genDtNcsModel.m to generate a discrete-time 
%   model of the NCS, then inputs the DT model into genPolyOvrAprx.m to
%   create a polytopic overapproximation and, finally, inputs the
%   overapproximated model into analyzePolyOvrAprx.m to verify robust 
%   stability. Stability analysis TYPEs are based on:
% 
%     GNB   - [Donkers et. al., Trans. Autom. Control 2011]
%     JNF   - [Cloosterman et. al., Trans. Autom. Control 2009]
%     CH    - [Gielen et. al., Automatica 2010]
%     STOCH - [Antunes et. a., ACC 2011]
%
%   INPUTS
%   ------------------------------------------------------
%      NCS              - NCS object created with the ncsEditor
%      TYPE             - 'GNB', 'JNF', 'CH' or 'STOCH'
%      QUANTVARS        - Strucuture containing additonal variables 
%                         needed for quantization analysis.  Input [] if
%                         quantization is not considered. See the Master's
%                         Thesis of S.J.L.M. van Loon, University of Technology
%                         Eindhoven, 2011, for theory and more info.
%                         ~~~~Required Fields~~~~
%                         QUANTVARS.type         - 'log', 'uniform' or 'None'
%                         QUANTVARS.alpha3       - (log,uniform) exponential decay rate parameter
%                         QUANTVARS.alpha4       - (log) storgage function parameter
%                         QUANTVARS.sigma        - (log,uniform) storgage function parameter
%                         QUANTVARS.omegaMIN     - (uniform) min eigenvalue of Lypaunov canidate 
%                                                   over all nodes
%                         QUANTVARS.omegaMAX     - (uniform) max eigenvalue of Lypaunov canidate 
%                                                   over all nodes
%                         QUANTVARS.minimizeIss  - (uniform)(RR only) 1 if sigma is to be minimized 
%                                                   by sdp, 0 otherwise
%      EPSU             - (GNB only) Desired overapproximation tightness parameter 
%      GRIDPTMAX        - (GNB only) Maximum number of gridpoints
%
%   OUTPUTS
%   ------------------------------------------------------
%       STABLE      - 1 if stability can be guaranteed, 0 otherwise
%       POLYOVRAPRX - strucuture with fields containing information
%                     regarding the polytopic overapproximation used for 
%                     analysis.  Type 'help genDtNcsModel.m' for more information.
%
%   See also NCS, GENDTNCSMODEL, GENPOLYOVRAPRX, ANALYZEPOLYOVRAPRX
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% input error checking

    if strcmp(type,'JNF') || strcmp(type,'CH')
        if nargin ~= 3
            error('dncs_main.m: Only three inputs are required for JNF or CH: DTMDL, OVRAPRXTYPE and QUANTVARS.')
        end
    elseif strcmp(type,'GNB')
        if nargin ~=  5
            error('dncs_main.m: Incorrect number of input arguements for GNB.')
        end
    elseif strcmp(type,'STOCH')   
         if nargin ~=  2
            error('dncs_main.m: Incorrect number of input arguements for STOCH.')
        end       
    else
        error('dncs_main.m: Invalid overapproximation method specified, must be JNF, CH or GNB')
    end

%% -- Generate Discrete-Time Model using NCS parameters

     [DtMdl,GammaMatrix] = genDtNcsModel(ncs);
   
%% -- Create Polytopic Overapproximation using Discrete-Time Model

    if strcmp(type,'JNF') || strcmp(type,'CH')
        polyOvrAprx = genPolyOvrAprx(DtMdl,type);
    elseif strcmp(type,'GNB');
        if nargin == 5
            polyOvrAprx = genPolyOvrAprx(DtMdl,'GNB',epsU,gridPtMax,false);
        elseif nargin == 6
            polyOvrAprx = genPolyOvrAprx(DtMdl,'GNB',epsU,gridPtMax,false);
        end
    end
    
%% -- LMI based stability analysis using Polytopic Overapproximation
    
    if ~strcmp(ncs.getNetwork.quantizer,'none') && isempty(quantVars)
       error('quantVars must be specified when quantizer is present.') 
    end
        
     if strcmp(type,'STOCH');  
        [stable,StabInfo] = analyzeStochStability(ncs);
     else 
        [stable,StabInfo] = analyzePolyOvrAprx(polyOvrAprx,ncs.getNetwork.protocol,GammaMatrix,quantVars);
     end
    
    disp(['  ' StabInfo.info])
        if stable ==1
            disp('  Stability: Guaranteed')
        else
            disp('  Stability: Not Guaranteed')
        end

    fprintf('-------------------------------------------------------------- \n')
end
