classdef ncs 
%NCS       Networked Control System class - use the ncsEditor to create an
%          instance of this class.  Properties can be easily modified within 
%          custom m-files once an instance is created.
%
% ncsObj = NCS(Nu,Ny) creates a new NCS object with Nu input channels and
% Ny output channels.
%
%Class Functions (Methods)
%-------------------------------------------------------------------------
%get methods
%     getPlant                      - gets plant matrices
%     getController                 - gets controller matrices
%     getNetwork                    - gets parmeters related to the network
%     getDtNcsModel                 - generates matrices for the discrete-time model representation
%     getPolyOvrAprx                - generates a polytopic overapproximaion
%     evaluateDtNcsModel            - evaluates closed-loop "A" matrix at constant txIntvl, delay and node
%     plotTimingUncertaintyRegion   - plot of the timing uncertainty polygon/region
%
%set methods
%     setPlant              - change the plant matrices
%     setController         - change the controller matrices
%     setNetwork            - change a group of network parameters
%     genTPoly              - change the uncertain polygon used for robust analysis and simulation
%                             based on the timing histogram (if provided)
%
%other methods
%     isNcsStable           - verifies if stablity can be guaranteed
%     findMatiMadBoundry    - generates a (MATI,MAD) stability boundry
%     simulateNcs           - simulates the ncs
%===========================================================
%
%   See also NCSEDITOR
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%%
    properties (Access = private)
        nu          %number of inputs
        ny          %number of outputs
        
        A           %plant A matrix
        B           %plant B matrix
        C           %plant C matrix
        
        
        ctrlType    % type of controller
        K           % state feedback gain
        L           % output injection gain
        Ac          % dynamic feedback A matrix
        Bc          % dynamic feedback B matrix
        Cc          % dynamic feedback C matrix
        Dc          % dynamic feedback D matrix
        uWired      % 1 if controller is wired to actuators, 0 otherwise
        
        tau         % [taumin,taumax] bounds on the delay
        h           % [hmin,hmax] bounds on the transmission time
        tPoly       % uncertainty polygon
        timingDist  % probabiliy distribution of h and tau
        delta       % integer bound on the number of subsequent dropous
        gammaU      % matrix of 1's and 0's on the diagonal indicating network nodes
        gammaY      % matrix of 1's and 0's on the diagonal indicating network nodes
        protocol    % either TOD or RR
        quantizer   % quantizer used
    end
    
    methods
%% Constructor Function        
        function obj = ncs(nu,ny) 
            obj.nu = nu;
            obj.ny= ny;
            
            %default plant
            obj.A = 0;
            obj.B = zeros(1,nu);
            obj.C = zeros(ny,1);
            
            %default controller
            obj.ctrlType = 'Static Feedback';
            obj.K{1} = zeros(ny,nu);
            
            %default network
            obj.tau = [0 0];
            obj.h = [1 1];
            obj.tPoly=[];
            obj.timingDist=[];
            obj.delta = 0;
            obj.gammaU{1} = eye(nu);
            obj.gammaY{1} = eye(ny);
            obj.protocol = 'RR';
            obj.quantizer = 'none';
            
        end
%% Class Functions      
        
        
        %get methods
        plant       = getPlant(obj);
        controller  = getController(obj);
        network     = getNetwork(obj);
        dtNcsModel  = getDtNcsModel(obj);
        ovrAprx     = getPolyOvrAprx(obj,ovraprx,arg1,arg2,arg3);
        A           = evaluateDtNcsModel(obj,h,tau,sigma)

        %set methods
        obj = setPlant(obj,A,B,C);
        obj = setController(obj,arg0,arg1,arg2,arg3,arg4,arg5);
        obj = setNetwork(obj,network,value);
        obj = genUPloly(obj,delta,res);
        
        %other methods
        [stable,verts]  = isNcsStable(obj,ovraprx,arg1,arg2,arg3,arg4);
        [Hmati,Tmad]    = findMatiMadBoundry(obj,plotFlag);
        sim             = simulateNcs(obj,varargin);
                          plotTimingUncertaintyRegion(obj);
                          disp(obj);
        
        
    end % methods
      
end

