function obj = setController(obj,arg0,arg1,arg2,arg3,arg4,arg5)
% SETCONTROLLER Sets the plant of an NCS object. 
%
% NCS = SETCONTROLLER(NCS,K) sets the controller of the NCS object 
% such that the dynamics are
%
%         u = K{ell}*\hat{y}
%
% where ell correpsonds to the node which receives network access.
%
% NCS = SETCONTROLLER(NCS,'C-LTI',A,B,C,D) sets 
% the controller of the NCS object such that the dynamics are
%
%   \dot{z} = A{ell}*z + B{ell}*\hat{y}
%         u = C{ell}*\hat{y} + D{ell}*\hat{u}
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% NCS = SETCONTROLLER(NCS,'D-LTI',A,B,C,D) sets 
% the controller of the NCS object such that the dynamics are
%
%   z_{k+1} = A{ell}*z_k + B{ell}*hat{y}_k
%       u_k = C{ell}*z_k + D{ell}*\hat{u}_k
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% NCS = SETCONTROLLER(NCS,'D-LTI',A,B,C,K,L) sets 
% the controller of the NCS object such that the dynamics are
%
%   z_{k+1} = A{ell}*z_k + B{ell}*[  hat{u}_k  ] - L{ell}*(\hat{y}_k-C{ell}*z_k)
%                                 [hat{u}_{k-1}]
%       u_k = K{ell}*[    z_k     ]
%                    [hat{u}_{k-1}]
%
% where ell correpsonds to the node which receives network access and
% \hat{y} and \hat{u} are the 'networked versions' of y and u.
%
% NCS = SETCONTROLLER(NCS,CONTROLLER) can also be used, where CONTROLLER is the 
% structure resulting from the method GETCONTROLLER.
%
% NOTE: The dimensions of B{ell} and C{ell} must correspond to the number of inputs and
% outputs of the NCS object.
%
% See also GETCONTROLLER
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ------------------------------------------------------------------------- 

    if nargin == 2 && exist('arg0.ctrlType','var')
        if strcmp(arg0.ctrlType,'Static Feedback')

            %check if all fields exist
            if ~isfield(arg0,'K')
                error('K must be a field in the controller object')
            end
            
            if size(arg0{1}.K,1) ~= obj.ny
                error('Controller has a different number of inputs than plant has outputs.')
            elseif size(arg0{1}.K,2) ~= obj.nu
                error('Controller has a different number of outputs than plant has inputs.')
            end
            obj.ctrlType = arg0.ctrlType;
            obj.K = arg0.K;
        elseif strcmp(arg0.ctrlType,'C-LTI Dynamic Feedback') || strcmp(arg0.ctrlType,'D-LTI Dynamic Feedback' )

            %check if all fields exist
            if ~isfield(arg0,'A')
                error('A must be a field in the controller object')
            elseif ~isfield(arg0,'B')
                error('B must be a field in the controller object')
            elseif ~isfield(arg0,'C')
                error('C must be a field in the controller object')
            elseif ~isfield(arg0,'D')
                error('D must be a field in the controller object')
            end
            
            if size(arg0{1}.B,2) ~= obj.ny
                error('Controller has a different number of inputs than plant has outputs.')
            elseif size(arg0{1}.C,1) ~= obj.nu
                error('Controller has a different number of outputs than plant has inputs.')
            end
            
            obj.ctrlType = arg0.ctrlType;
            obj.Ac = arg0.A;
            obj.Bc = arg0.B;
            obj.Cc = arg0.C;
            obj.Dc = arg0.D;
        elseif strcmp(arg0.ctrlType,'D-LTI Observer-Based Feedback')

            %check if all fields exist
            if ~isfield(arg0,'A')
                error('A must be a field in the controller object')
            elseif ~isfield(arg0,'B')
                error('B must be a field in the controller object')
            elseif ~isfield(arg0,'C')
                error('C must be a field in the controller object')
            elseif ~isfield(arg0,'K')
                error('K must be a field in the controller object')
            elseif ~isfield(arg0,'L')
                error('L must be a field in the controller object')
            end
            
            if size(arg0{1}.B,2) ~= 2*obj.nu
                error('The B matrix should have twice as many columns as the plant has inputs.')
            elseif size(arg0{1}.C,1) ~= obj.ny
                error('The C matrix should have the same number of rows as the plant has outputs.')
            elseif size(arg0{1}.K,1) ~= obj.nu+size(obj.getPlant.A,1)
                error('The K matrix should have nx+nu columns where nx is the number of plant states and nu is the number of inputs.')                
            end
            
            obj.ctrlType = arg0.ctrlType;
            obj.Ac = arg0.A;
            obj.Bc = arg0.B;
            obj.Cc = arg0.C;
            obj.Dc = arg0.D;
            obj.K = arg0.K;
            obj.L = arg0.L;           
        else
            error('wrong input')
        end
    elseif nargin == 2 && ~exist('arg0.ctrlType','var')    
        if ~iscell(arg0)
            obj.ctrlType = 'Static Feedback';
            obj.K{1} = arg0;
        elseif size(arg0{1},1) ~= obj.nu
            error('Controller has a different number of outputs than plant has inputs.')
        elseif size(arg0{1},2) ~= obj.ny
            error('Controller has a different number of inputs than plant has outputs.')
        else
            obj.ctrlType = 'Static Feedback';
            obj.K = arg0; 
        end
        
        
    elseif nargin == 6
        
        if ~iscell(arg1) && ~iscell(arg2) && ~iscell(arg3) && ~iscell(arg4)
            arg1={arg1};
            arg2={arg2};
            arg3={arg3};
            arg4={arg4};
        end
        
        if size(arg2{1},2) ~= obj.ny
            error('Controller has a different number of inputs than plant has outputs.')
        elseif size(arg3{1},1) ~= obj.nu
            error('Controller has a different number of outputs than plant has inputs.')
        end
        if strcmp(arg0, 'C-LTI')
            obj.ctrlType='C-LTI Dynamic Feedback';
        elseif strcmp(arg0, 'C-LTI uWired')
            obj.ctrlType='C-LTI Dynamic Feedback (uWired)';
        elseif strcmp(arg0,'D-LTI')
            obj.ctrlType='D-LTI Dynamic Feedback';
        end
        obj.Ac=arg1;
        obj.Bc=arg2;
        obj.Cc=arg3;
        obj.Dc=arg4;
    elseif nargin == 7
        if ~iscell(arg1) && ~iscell(arg2) && ~iscell(arg3) && ~iscell(arg4)
            arg1={arg1};    %A
            arg2={arg2};    %B
            arg3={arg3};    %C
            arg4={arg4};    %K
            arg5={arg5};    %L
        end

        if size(arg2{1},2) ~= 2*obj.nu
            error('The B matrix should have twice as many columns as the plant has inputs.')
        elseif size(arg4{1},2) ~= obj.nu+size(obj.getPlant.A,1)
            error('The K matrix should have nx+nu columns where nx is the number of plant states and nu is the number of inputs.')                
        end

        if strcmp(arg0, 'C-LTI')
            error('Not implemented yet.  Use D-LTI.')
        elseif strcmp(arg0,'D-LTI')
            obj.ctrlType='D-LTI Observer-Based Feedback';
        end
        
        obj.Ac=arg1;
        obj.Bc=arg2;
        obj.Cc=arg3;
        obj.K=arg4;
        obj.L=arg5;
    else
        error('wrong input')
    end

end