function obj = setNetwork(obj, net_property, value)
% SETNETWORK Returns the net_property parameters of an NCS object. 
%
% NOTE: It is easiest to initially set net_property parameters from the NCS
% Editor GUI.  This command line function is most useful for modification 
% purposes.
%
% NCS = SETNETWORK(NCS,net_property) sets the all net_property parameters.  The 
% structure net_property must have the following fields:
%
%   tPoly      = list of vertices which define polygon that refine the uncertain region
%   timingDist = timing distribution used for stochastic anaylsis
%   maxDrops   = integer indicating the max drops possible
%   nodesU     = set of matrices indicating which plant inputs are associated
%                with which node number
%   nodesY     = set of matrices indicating which plant outputs are associated
%                with which node number
%   protocol   = 'RR' for Round Robin or 'TOD' for Try-Once-Discard protocol
%   quanitzer  = 'none', 'uniform', 'log' specifying type of quanizer used
%
% The structure net_property is ouput from GETNETWORK. It is easiest to call
% GETNETWORK and then modify the stucture and then SETNETWORK.
%
% NCS = SETNETWORK(NCS,'tPoly',VERTS) sets only the uncertainty region
% bounds where VERTS is either a list of (txIntl,delay) vertices specifying 
% or a the timing uncertainty polygon or a vector 
% [txIntvl_min txIntvl_max delay_min delay_max] which will be used to create 
% a timing uncertainty polygon
%
% NCS = SETNETWORK(NCS,'timingDist',PDF) sets only the timing distrbution
% where PDF is a list of (txIntl,delay,probability(in %)) entries
%
% NCS = SETNETWORK(NCS,'maxDrops',BOUND) sets only the dropout
% bounds where BOUND is a non-negative integer
%
% NCS = SETNETWORK(NCS,'nodesU',NODES) sets only the input nodes
% where NODES is a cell matrices which specify which inputs belong to
% different nodes
%
% NCS = SETNETWORK(NCS,'nodesY',NODES) sets only the output nodes
% where NODES is a cell matrices which specify which outputs belong to
% different nodes
%
% NCS = SETNETWORK(NCS,'protocol',TYPE) sets only the dropout
% bounds where TYPE is the type of protocol
%
% NCS = SETNETWORK(NCS,'quanitzer',TYPE) sets only the dropout
% bounds where TYPE is the type of quanitizer
%
% See also GETNETWORK
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if nargin==2 
        %check if all fields exist in structure input
        if ~isfield(net_property,'maxDrops')
            error('maxDrops must be a field in the net_property object')
        elseif ~isfield(net_property,'tPoly')
            error('tPoly must be a field in the net_property object')
        elseif ~isfield(net_property,'nodesU')
            error('nodesU must be a field in the net_property object')
        elseif ~isfield(net_property,'nodesY')
            error('nodesY must be a field in the net_property object')
        elseif ~isfield(net_property,'protocol')
            error('protocol must be a field in the net_property object')
        elseif ~isfield(net_property,'quantizer')        
            error('quantizer must be a field in the net_property object')
        end
    end
    
validAttribute = 0;

%%%%%%%%%%%%%%%%%%%%%%%  
% Error Checking for individual attributes  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if nargin==2
         value = net_property.maxDrops;   
    end
    if strcmp(net_property,'maxDrops') || nargin==2
        validAttribute = 1;
        maxDrops = value;
        if ~isnumeric(value)
            error('Max drops must be numeric.')
        elseif value<0
            error('Max drops must be positive.')
        elseif round(value)~=value
            error('Max drops must be an integer.')
        end

        obj.delta = maxDrops;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if nargin==2
         value = net_property.tPoly;   
    end    
    if strcmp(net_property,'tPoly') || nargin==2
        validAttribute = 1;

        if ~isempty(value)
            if(size(value,2)==2)
                if any((value(:,1)<value(:,2)))
                    error('This toolbox does not support cases where txIntvl < delay. tPoly points do not satisfy small delay assumption');
                end
                %set hmin,hmax,taumin,taumax based on tPoly
                obj.h   = [min(value(:,1)) max(value(:,1))];
                obj.tau = [min(value(:,2)) max(value(:,2))];
                obj.tPoly = value;
            elseif sum(size(value)==[1 4])==2
                bounds = [value(1) value(2) value(3) value(4)];
                err = checkTimingBounds(bounds);
                
                if ~err
                    %generate polygon from hmin,hamx,taumin,taumax
                    obj.tau = [value(3) value(4)];
                    obj.h   = [value(1) value(2)];
                    obj.tPoly = genTimingUncertaintyRegion(value(1),value(2),value(3),value(4));
                else
                    error(err);
                end

            else
                error('tPoly must be a list of vertices, each only 2 elements [txIntvl delay].');
            end
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if nargin==2
         value = net_property.timingDist;   
    end    
    if strcmp(net_property,'timingDist') || nargin==2
        validAttribute = 1;
        if ~isempty(value)
           if size(value,2)~=3
                error('timingDist must be a list of only 3 elements: vertices with probabiliy [txIntvl delay prob].')
           elseif abs(sum(value(:,3))-100)>1e-4
                error('Third column indicates the probability in percent and should sum to 100.')
           elseif any(value(:,1)<value(:,2))
                error('This toolbox does not support cases where txIntvl < delay. One or more of the [txIntvl delay] combinations does not satisfy this condition.')
           end
        end
        obj.timingDist=value;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if nargin==2
         value = net_property.nodesU;   
    end
    if strcmp(net_property,'nodesU') || nargin==2
        validAttribute = 1;
        if ~iscell(value)
            error('nodesU must be a cell.')
        end
        for ctr=1:length(value)
            if size(value{ctr},1)~=size(value{ctr},2)
                error('nodesU must be a square matrix')
            end
            if size(value{ctr},1)~=size(obj.getPlant.B,2)
                error('nodesU dimension does not match number of inputs')
            end
            
            for row=1:size(value{ctr},1)
                ind=find(value{ctr}(row,:)~=0);
                if length(ind)>1
                    error('nodesU containts a nonzero off diagonal value')
                elseif ~isempty(ind) && ind~=row
                    error('nodesU containts a nonzero off diagonal value')
                elseif ~isempty(ind) && value{ctr}(ind,ind)~=1
                    error('nodesU contains a nonzero value that is not equal to one.')
                end
            end
        end
        obj.gammaU=value;
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if nargin==2
         value = net_property.nodesY;   
    end
    if strcmp(net_property,'nodesY') || nargin==2
        validAttribute = 1;
        if ~iscell(value)
            error('nodesY must be a cell.')
        end
        for ctr=1:length(value)
            if size(value{ctr},1)~=size(value{ctr},2)
                error('nodesY must be a square matrix')
            end
            if size(value{ctr},1)~=size(obj.getPlant.C,1)
                error('nodesY dimension does not match number of outputs')
            end
            
            for row=1:size(value{ctr},1)
                ind=find(value{ctr}(row,:)~=0);
                if length(ind)>1
                    error('nodesY containts a nonzero off diagonal value')
                elseif ~isempty(ind) && ind~=row
                    error('nodesY containts a nonzero off diagonal value')
                elseif ~isempty(ind) && value{ctr}(ind,ind)~=1
                    error('nodesY contains a nonzero value that is not equal to one.')
                end
            end
        end
        obj.gammaY=value;
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if nargin==2
         value = net_property.protocol;   
    end
    if strcmp(net_property,'protocol') || nargin==2
        validAttribute = 1;
        if ~strcmp(value,'RR') && ~strcmp(value,'TOD')
            error('protocol type not recognized, must be either RR or TOD.')
        end   
        obj.protocol=value;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if nargin==2
         value = net_property.quantizer;   
    end   
    if strcmp(net_property,'quantizer') || nargin==2
        validAttribute = 1;
        if ~strcmp(value,'none') && ~strcmp(value,'log') && ~strcmp(value,'uniform')
            error('quanitzer type not recognized, must be either none, log or uniform.')
        end
        obj.quantizer=value;
    end

    if validAttribute==0
       error('Network property specified is unknown.') 
    end
    
end