function sim = simulateNcs(obj,varargin)
% SIMULATENCS simulates an NCS with given inital conditions. Time
% trajectories of the closed-loop states, input and output signals, and
% network timing characteristics are plotted.
%
% SIMULATENCS(NCS[,DROP_PROB]) simulates an NCS for T=10 seconds and the inital 
% states of the closed-loop NCS (X0) are such that all plant 
% states are equal to 1 and all other closed-loop states are equal to zero. 
% The Tx intervals, delays and dropouts are randomly generated with uniform
% distribution. Dropout probability DROP_PROB can be specified.
%
% SIMULATENCS(NCS,X0,T[,DROP_PROB]) simulates an NCS for T seconds and the inital 
% states of the closed-loop NCS (X0) are user specified and the Tx intervals, 
% delays and dropouts are randomly generated with uniform distribution. Dropout
% probability DROP_PROB can be specified.
%
% SIMULATENCS(NCS,X0,TXINTVLS,DELAYS,NODE_SEQ) simulates an NCS for T seconds,  
% the inital states of the closed-loop NCS (X0) user specified and the Tx 
% intervals, delays and node sequence are also user specified.
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if isfield(obj.getController,'A')
        if length(obj.getController.A)>1 && ~strcmp(obj.getController.ctrlType,'Static Feedback')

           %need to extract the u,y signals 
           error('Simulation of node dependent controllers have not been implemented yet.') 
        end
        
    end
    
    if isfield(obj.getController,'K')
        if length(obj.getController.K)>1 && strcmp(obj.getController.ctrlType,'Static Feedback')
            %need to extract the u,y signals 
            error('Simulation of node dependent controllers have not been implemented yet.') 
        end
    end
    
    nx = size(obj.getPlant.A,1);
    ney = size(obj.getPlant.C,1);
    
    if ~strcmp(obj.getController.ctrlType,'Static Feedback')
        nc = size(obj.getController.A{1},1);
    else
        nc=0;
    end
    
    if ~strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        neu = size(obj.getPlant.B,2);
    else
        neu=0;
    end
    
    
    nCL=nx+nc+ney+neu;
    nrNodes=length(obj.getNetwork.nodesU);
    
    DtNcsModel = cell(1,nrNodes);
    Gamma = cell(1,nrNodes);
    
    %Create a DtNcsModel for each node 
    for ell=1:nrNodes
        net=obj.getNetwork;
        net.nodesU=net.nodesU(ell);
        net.nodesY=net.nodesY(ell);
        obj1 = obj.setNetwork(net);
        [DtNcsModel{ell} gamma]=genDtNcsModel(obj1,0);
        Gamma{ell}=gamma{1};
        DtNcsModel{ell}.tPoly=[];     % so overapproximation does not happen
    end
    
    net=obj.getNetwork;
    net.nodesU={0.*net.nodesU{ell}};
    net.nodesY={0.*net.nodesY{ell}};
    obj1 = obj.setNetwork(net);
    DtNcsModel_dropped=genDtNcsModel(obj1,0);
    DtNcsModel_dropped.tPoly=[];     % so overapproximation does not happen
    
    %if DROP_PROB isn't specified, set it to 0.5
    if nargin==1
        varargin{1}=0.5;
        T=100;
        x=zeros(nCL,T);
        
        x(1:nx,1)= ones(nx,1);
        x(nx+1:nCL,1)= zeros(ney+neu+nc,1); 
        
        % define network signals for simulation 
        [h tau]= generateTimingSequence(obj,T+1);
        delta  = generateDropSequence(obj,T+1,varargin{1});
    elseif nargin==2
        %Tsec=10;
        %T=ceil(Tsec/((net.txIntvls(2)+net.txIntvls(1))/2));
        T=100;
        x=zeros(nCL,T);
        
        x(1:nx,1)= ones(nx,1);
        x(nx+1:nCL,1)= zeros(ney+neu+nc,1);
        
        % define network signals for simulation 
        [h tau]= generateTimingSequence(obj,T+1);
        delta  = generateDropSequence(obj,T+1,varargin{1});
    elseif nargin==3
        varargin{3}=0.5;
        Tsec=varargin{2};
        T=ceil(Tsec/((net.txIntvls(2)+net.txIntvls(1))/2));
        
        x=zeros(nCL,T);
        x(:,1)=varargin{1};
        
        % define network signals for simulation 
        
        [h tau]=generateTimingSequence(obj,T+1);
        delta=generateDropSequence(obj,T+1,varargin{3});
    elseif nargin==4
        Tsec=varargin{2};
        T=ceil(Tsec/((net.txIntvls(2)+net.txIntvls(1))/2));
        
        x=zeros(nCL,T);
        x(:,1)=varargin{1};
        
        % define network signals for simulation 
        
        [h tau]=generateTimingSequence(obj,T+1);
        delta=generateDropSequence(obj,T+1,varargin{3});
    elseif nargin==5
        T=length(varargin{2});
        x=zeros(nCL,T);
        x(:,1)=varargin{1};
        
        h=varargin{2};
        tau=varargin{3};
        sigma=varargin{4};
        delta = zeros(1,length(sigma));
    end
    
    %------------------------------------------------------------------
    %------------------------------------------------------------------
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Simulate NCS Evolution
    fprintf('      Simulation Progress:  %u %% complete',0)
    t=zeros(1,T);
    for k=1:T+1
        fprintf(1, repmat('\b',1,13)); %delete line before
        fprintf('%3u%% complete',uint8(k/(T+1)*100))        
        
        %determine which node transmits
        if nargin ~= 5
            if k>1 && delta(k)==1
                sigma(k)=sigma(k-1);    %in case of drop, same node keeps retrying to transmit
            else
            
                if strcmp(obj.getNetwork.protocol,'RR')
                    if k==1
                        sigma(1)=1;
                    elseif sigma(k-1)+1<=nrNodes
                        sigma(k)=sigma(k-1)+1;
                    else
                        sigma(k)=1;
                    end
                elseif strcmp(obj.getNetwork.protocol,'TOD')
                    err=zeros(1,nrNodes);
                    for ell=1:nrNodes
                        e=x(nx+nc+1:end,k);
                        err(ell)=e'*Gamma{ell}*e;
                    end
                    [err_max sigma(k)]=max(err);
                end
            end
        end
        
        if delta(k)==1
            DtNcsModel_dropped.tPoly=[h(k) tau(k)];
            polyOvrAprx=genPolyOvrAprx(DtNcsModel_dropped,'GNB',1,1,0,0);
        else
            DtNcsModel{sigma(k)}.tPoly=[h(k) tau(k)];
            polyOvrAprx=genPolyOvrAprx(DtNcsModel{sigma(k)},'GNB',1,1,0,0);
        end
        
        t(k+1)=t(k)+h(k);
        x(:,k+1) = polyOvrAprx.Ab{1}*x(:,k);
        
    end
    disp(' ')    
    %------------------------------------------------------------------
    %------------------------------------------------------------------
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %Extract Data From Model
    
    bounds.txIntvl  = net.txIntvls;
    bounds.delay    = net.delays;
    r=t(1:end-1)+tau; 
    
    % calculate effective (h,tau) point cause by drops occurring
    ctr=1;
    ctr2=1;
    while ctr<length(h)
       h_eff(ctr2)=h(ctr);
       t_eff(ctr2)=t(ctr);
       while ctr<length(h) && delta(ctr+1)==1  %if next one is a drop
           ctr=ctr+1;
           h_eff(ctr2)=h_eff(ctr2)+h(ctr);     %add transmision interval
       end
       tau_eff(ctr2)=tau(ctr);
       ctr2=ctr2+1;
       ctr=ctr+1;
    end
       
    
    %Extract State Evolution
    if strcmp(obj.getController.ctrlType,'Static Feedback')        
        xp=x(1:nx,:);
        xc=zeros(1,T);  % controller does not have an internal state
        ey=x(nx+1:nx+ney,:);
        eu=x(nx+ney+1:nCL,:);
    elseif strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        xp=x(1:nx,:);
        xc=x(nx+1:nx+nc,:);
        ey=x(nx+nc+1:nCL,:);
        eu =zeros(1,T);    % no eu state
    else
        % controller has an internal state
        xp=x(1:nx,:);
        xc=x(nx+1:nx+nc,:);
        ey=x(nx+nc+1:nx+nc+ney,:);
        eu=x(nx+nc+ney+1:nCL,:);
    end    
    
    %determine y, yhat
    y=obj.getPlant.C*xp;
    haty_min1=y+ey;
    haty=haty_min1(:,2:end);

    %determine u, uhat
    if strcmp(obj.getController.ctrlType,'Static Feedback')
        u=-obj.getController.K{1}*haty;
    elseif strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback') ||...
            strcmp(obj.getController.ctrlType,'D-LTI Dynamic Feedback') ||...
             strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        u=obj.getController.C{1}*xc+obj.getController.D{1}*haty_min1;
    elseif strcmp(obj.getController.ctrlType,'D-LTI Observer-Based Feedback')
        Kx=obj.getController.K{1}(:,1:nx);
        Ku=obj.getController.K{1}(:,1+nx:end);
        hatu_min1=(eye(neu)-Ku)\(eu+Kx*xc);
        u=Kx*xc+Ku*hatu_min1;
    end

    if ~strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        hatu=u(:,2:T+1)+eu(:,2:T+1);
    else
        hatu=zeros(1,T);
    end
    
    %------------------------------------------------------------------
    %------------------------------------------------------------------
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % construct data used for plotting
    
    sim.states.xp = [t(:,1:T)' xp(:,1:T)'];
    sim.states.xc = [t(:,1:T)' xc(:,1:T)'];
    sim.states.eu = [t(:,1:T)' eu(:,1:T)'];
    sim.states.ey = [t(:,1:T)' ey(:,1:T)'];
    
    sim.signals.u    = [t(:,1:T)'    u(:,1:T)'];
    sim.signals.uhat = [r(:,1:T)' hatu(:,1:T)'];
    sim.signals.y    = [t(:,1:T)'    y(:,1:T)'];   % y_k updates at transmission times 't'
    sim.signals.yhat = [r(:,1:T)' haty(:,1:T)'];   % haty_k updates at reception times 'r'
    
    sim.network.txIntvls    = [t_eff'    h_eff'];
    sim.network.delays      = [t_eff'    tau_eff'];
    sim.network.nodeSeq     = [t(:,1:T)'        sigma(:,1:T)'];
    sim.network.drops       = [t(:,1:T)'        delta(:,1:T)'];    
    
    %------------------------------------------------------------------
    %------------------------------------------------------------------
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Plot ALL States and Signals
    
    %---------------------------
    %for arrange figures 
    f1=figure;
    pos1=get(f1,'Position');
    scale = 0.8;
    width   = scale*pos1(3);
    height  = scale*pos1(4);
    set(f1,'Position',[pos1(1:2) width height])
    opos1 = get(f1,'OuterPosition');    
    oWidth  = opos1(3);
    oHeight = opos1(4);
    
    srn = get(0,'ScreenSize');
    centerX = srn(3)/2;
    centerY = srn(4)/2;
    %---------------------------
    
    %% Plot State Evolution
    set(f1,'Position',[centerX-oWidth   centerY          width height]);
    
    if strcmp(obj.getController.ctrlType,'Static Feedback')

        sub1=subplot(3,1,1);
        title('plant state evolution')
        hold all;
        for n=1:nx
            stairs(sim.states.xp(:,1), sim.states.xp(:,n+1));
        end
        ylabel('x_p')
        %%%
        sub2=subplot(3,1,2);
        title('controller state evolution')
        hold all;
        for n=1:ney
            stairs(sim.states.ey(:,1), sim.states.ey(:,n+1));
        end
        ylabel('e^y')
        %%%
        sub3=subplot(3,1,3);
        title('outputs network-induced error evolution')
        hold all;
        for n=1:neu
            stairs(sim.states.eu(:,1), sim.states.eu(:,n+1));
        end
        ylabel('e^u')
        xlabel('time (s)')    
        linkaxes([sub1 sub2 sub3],'x')
    elseif strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        
        sub1=subplot(3,1,1);
        title('plant state evolution')
        hold all;
        for n=1:nx
            stairs(sim.states.xp(:,1), sim.states.xp(:,n+1));
        end
        ylabel('x_p')
        %%%
        sub2=subplot(3,1,2);
        title('controller state')
        hold all;
        for n=1:nc
            stairs(sim.states.xc(:,1), sim.states.xc(:,n+1));
        end
        ylabel('x_c')
        %%%
        sub3=subplot(3,1,3);
        title('controller state evolution')
        hold all;
        for n=1:ney
            stairs(sim.states.ey(:,1), sim.states.ey(:,n+1));
        end
        ylabel('e^y')
        xlabel('k')    
        linkaxes([sub1 sub2 sub3],'x')
    else
        % controller has an internal state
        sub1=subplot(4,1,1);
        title('plant state')
        hold all;
        for n=1:nx
            stairs(sim.states.xp(:,1), sim.states.xp(:,n+1));
        end
        ylabel('x_p')
        %%%
        sub2=subplot(4,1,2);
        title('controller state')
        hold all;
        for n=1:nc
            stairs(sim.states.xc(:,1), sim.states.xc(:,n+1));
        end
        ylabel('x_c')
        %%%
        sub3=subplot(4,1,3);
        title('input network-induced error state')
        hold all;
        for n=1:ney
            stairs(sim.states.ey(:,1), sim.states.ey(:,n+1));
        end
        ylabel('e^y')
        %%%
        sub4=subplot(4,1,4);
        title('output network-induced error state')
        hold all;
        for n=1:neu
            stairs(sim.states.eu(:,1), sim.states.eu(:,n+1));
        end
        ylabel('e^u')
        xlabel('time (s)')
        linkaxes([sub1 sub2 sub3 sub4],'x')
    end
    
    %% Plot input/output Signal Evolution
    figure('Position',[centerX          centerY          width height]);
    color=[0 0 255;
           0 127 0;
           255 0 0;
           0 191 191;
           191 0 191;
           191 191 0;
           63 63 63]./255;
    if max(size(u,1),size(y,1))>size(color,1)
        repmat(color,ceil(max(size(y,1),size(u,1))/length(color)),1);
    end
    
    sub1=subplot(2,1,1);
    ylabel('output signals')
    hold all;
    for ctr=1:size(y,1)
        h1=stairs(sim.signals.y(:,1),sim.signals.y(:,ctr+1),'Color',color(ctr,:));
        h2=stairs(sim.signals.yhat(:,1),sim.signals.yhat(:,ctr+1),':','Color',color(ctr,:));
    end
    legend([h1,h2],'y_k','yHat_k')
    
    sub2=subplot(2,1,2);
    ylabel('input signals')       
    hold all;
    for ctr=1:size(u,1)
        h1=stairs(sim.signals.u(:,1),sim.signals.u(:,ctr+1),'Color',color(ctr,:));
        if ~strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
            h2=stairs(sim.signals.uhat(:,1),sim.signals.uhat(:,ctr+1),':','Color',color(ctr,:));
        end
    end
    
    if ~strcmp(obj.getController.ctrlType,'C-LTI Dynamic Feedback (uWired)')
        legend([h1,h2],'u_k','uHat_k');
    else
        legend(h1,'u_k');
    end
    
    
    xlabel('time (s)')
    linkaxes([sub1 sub2],'x')

    %% Plot bounded uncertainty regions for different dropout levels
    figure('Position',[centerX          centerY-oHeight  width height]);
    hold all;
    for ctr=0:net.maxDrops
          if ctr==0
              if ~isempty(net.tPoly)
                r0=net.tPoly;
                if polyarea(r0(:,1),r0(:,2))
                  k=convhull(r0);
                  r0=r0(k,:);
                end
                h2=patch(r0(:,1),r0(:,2),'r','EdgeColor','r');
                ri=r0;
              end
          else
              if ~isempty(net.tPoly)
                  ri= findNextDropPoly(r0,ri);
                  h2=patch(ri(:,1),ri(:,2),'r','EdgeColor','r');         
              end
          end
    end
    alpha(0.3);
    h3=plot([0 max([tau_eff h_eff])*1.1],[0 max([tau_eff h_eff])*1.1],'k--');
    h4=plot(sim.network.txIntvls(:,2),sim.network.delays(:,2),'g.');
    
    title('(txInvl,delay) points tested - Dropouts modeled as a prolongation of the txIntvl')
    xlabel('transmission intervals (s)')
    ylabel('delays (s)')
    legend([h2,h3,h4],{'uncertain region boundry (incl. drops)','small-delay boundry','points used in simulation'})
    
    %% Network Timing Signals
    figure('Position',[centerX-oWidth   centerY-oHeight  width height]);
    if nrNodes>1
        %%%
        sub1=subplot(4,1,1);
        stairs(sim.network.txIntvls(:,1), sim.network.txIntvls(:,2));
        if bounds.txIntvl(2)-bounds.txIntvl(1)>1e-6
            hold all;
            plot([0 max(t)],[bounds.txIntvl(1) bounds.txIntvl(1)],'r--')
            plot([0 max(t)],[bounds.txIntvl(2) bounds.txIntvl(2)],'r--')
        end
        ylabel('txIntvl (s)')
        %%%
        sub2=subplot(4,1,2);
        %stairs(t_eff,tau_eff);
        stairs(sim.network.delays(:,1), sim.network.delays(:,2));
        if bounds.delay(2)-bounds.delay(1)>1e-6
            hold all;
            plot([0 max(t)],[bounds.delay(1) bounds.delay(1)],'r--')
            plot([0 max(t)],[bounds.delay(2) bounds.delay(2)],'r--')
        end
        ylabel('delay (s)')
        %%%
        sub3=subplot(4,1,3);
        %stairs(t,sigma);
        stairs(sim.network.nodeSeq(:,1), sim.network.nodeSeq(:,2));
        ylabel('node')
        axis([0 max(t) 0.5 nrNodes+0.5])
        %%%
        sub4=subplot(4,1,4);
        %stairs(t,delta);
        stairs(sim.network.drops(:,1), sim.network.drops(:,2));
        ylabel('drop')
        axis([0 max(t) -0.5 1.5])

        xlabel('time (s)')
        linkaxes([sub1 sub2 sub3 sub4],'x')
    else
        %%%
        sub1=subplot(3,1,1);
        %stairs(t_eff,h_eff);
        stairs(sim.network.txIntvls(:,1), sim.network.txIntvls(:,2));
        if bounds.txIntvl(2)-bounds.txIntvl(1)>1e-6
            hold all;
            plot([0 max(t)],[bounds.txIntvl(1) bounds.txIntvl(1)],'r--')
            plot([0 max(t)],[bounds.txIntvl(2) bounds.txIntvl(2)],'r--')
        end
        ylabel('txIntvl (s)')
        %%%
        sub2=subplot(3,1,2);
        %stairs(t_eff,tau_eff);
        stairs(sim.network.delays(:,1), sim.network.delays(:,2));
        if bounds.delay(2)-bounds.delay(1)>1e-6
            hold all;
            plot([0 max(t)],[bounds.delay(1) bounds.delay(1)],'r--')
            plot([0 max(t)],[bounds.delay(2) bounds.delay(2)],'r--')
        end
        ylabel('delay (s)')
        %%%
        sub3=subplot(3,1,3);
        %stairs(t,delta);
        stairs(sim.network.drops(:,1), sim.network.drops(:,2));
        ylabel('drop')
        axis([0 max(t) -0.5 1.5])

        xlabel('time (s)')
        linkaxes([sub1 sub2 sub3],'x')
        
    end
    
end


function [h tau] = generateTimingSequence(obj,T)
        % create some h,tau combinations that are within polygon
        
        hmin    = obj.getNetwork.txIntvls(1);
        hmax    = obj.getNetwork.txIntvls(2);
        taumin  = obj.getNetwork.delays(1);
        taumax  = obj.getNetwork.delays(2);

        if hmax-hmin<1e-12 && taumax-taumin<1e-12
            %constant txIntvl and constant delay
            h   = hmax*ones(1,T);
            tau = taumax*ones(1,T);
        elseif hmax-hmin<1e-12
            %varying txIntvl and constant delay
            h   = hmax*ones(1,T);
            tau = taumin + rand(1,T)*(taumax-taumin);
        elseif taumax-taumin<1e-12
            %constant txIntvl and varying delay
            h   = hmin + rand(1,T)*(hmax-hmin);
            tau = taumax*ones(1,T);
        else
            %varying txIntvl and varying delay

            %define polygon
            if isempty(obj.getNetwork.tPoly)
                poly=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
            else
                %get polygon
                poly=obj.getNetwork.tPoly;
                
                %create a convex hull of polygon
                k=convhull(poly);
                poly=poly(k,:);
                
                %create bounding box for custom polygon
                hmin = min(poly(:,1));
                hmax = max(poly(:,1));
                taumin = min(poly(:,2));
                taumax = max(poly(:,2));
            end
            hVerts=poly(:,1);
            tauVerts=poly(:,2);

            %create some random testpoints within the uncertain region
            h_test  = hmin   +rand(1,T)*(hmax-hmin);
            tau_test= taumin +rand(1,T)*(taumax-taumin);

            %check whether they are within the polygon
            in=inpolygon(h_test,tau_test,hVerts,tauVerts);

            %if they are within, keep them
            h=h_test(in);
            tau=tau_test(in);
            
            %--------------------------------------
            %testing code
            %figure;
            %patch(poly(:,1),poly(:,2),'k');
            %hold all;
            %plot(h,tau,'g.');
            %plot(h_test,tau_test,'rx');
            %--------------------------------------
            
            %keep adding random h,tau points within the polygon
            while length(h)<T
                h_test  = hmin   +rand(1,T)*(hmax-hmin);
                tau_test= taumin +rand(1,T)*(taumax-taumin);

                in=inpolygon(h_test,tau_test,hVerts,tauVerts);
                h=[h h_test(in)];
                tau=[tau tau_test(in)];
            end    

            h=h(1:T);
            tau=tau(1:T);
        end
end

function delta = generateDropSequence(obj,T,dropProb)

    deltaBar=obj.getNetwork.maxDrops;
    delta = zeros(1,T);

    if deltaBar~=0
        dropCtr=0;
        for ctr=1:T
            if dropCtr<deltaBar
                delta(ctr)= rand(1)<=dropProb; 
            else
                delta(ctr)=0;
            end

           if delta(ctr)==0
               dropCtr=0;
           else
               dropCtr=dropCtr+1;
           end

        end
    end
    
end

