function [stable, StabInfo, LMI] = analyzePolyOvrAprx(polyOvrAprx,analysisType,GammaMatrix,quantVars)
%ANALYZEPOLYOVRAPRX  Verifies LMI conditions to determine if a polytopic model 
%                    with norm bounded uncertainty is robustly stable.
%
%   [STABLE,STABINFO,LMI] = ANALYZEPOLYOVRAPRX(OVRAPRX,PROTOCOL,GAMMAMATRIX,QUANTVARS)
%   Verifies LMI conditions to determine if robust stability STABLE can be guaranteed 
%   of a polytopic model with norm bounded uncertainty OVRAPRX.  Two types
%   of protocols can be anaylzed, the Round Robin protocol (RR) and the Try
%   Once Discard (TOD) protocol.  For more information, see
%   [Donkers et. al., Trans. Autom Control 2011] and the Master's Thesis 
%   of S.J.L.M. van Loon, University of Technology Eindhoven, 2011.
%                     
%
%   INPUTS
%   ------------------------------------------------------
%      POLYOVRAPRX  - strucutre from genPolyOvrAprx.m.  The required
%                     fields are Ab,Bb,Cb,Db,Eb,Jb and JordanBlks.
%                     Type 'help genPolyOvrAprx.m' to see a full list of
%                     properties.
%      ANALYSISTYPE - 'RR' or 'TOD'
%      GAMMAMATRIX  - (TOD) Strucure containing gamma matrices associated with protocol 
%      QUANTVARS    - (Optional) Strucuture containing additonal variables 
%                     needed for quantization analysis.  See the Master's
%                     Thesis of S.J.L.M. van Loon, University of Technology
%                     Eindhoven, 2011, for theory and more info.
%                     ~~~~Required Fields~~~~
%                     QUANTVARS.type         - 'log', 'uniform' or 'None'
%                     QUANTVARS.alpha3       - (log,uniform) exponential decay rate parameter
%                     QUANTVARS.alpha4       - (log) storgage function parameter
%                     QUANTVARS.sigma        - (log,uniform) storgage function parameter
%                     QUANTVARS.omegaMIN     - (uniform) min eigenvalue of Lypaunov canidate 
%                                               over all nodes
%                     QUANTVARS.omegaMAX     - (uniform) max eigenvalue of Lypaunov canidate 
%                                               over all nodes
%                     QUANTVARS.minimizeIss  - (uniform)(RR only) 1 if sigma is to be minimized 
%                                               by sdp, 0 otherwise
%
%   OUTPUTS
%   ------------------------------------------------------
%       STABLE      - 1 if stability can be guaranteed, 0 otherwise
%       STABINFO    - output from SeDuMi
%       LMI         - set of LMIs input into solvesdp()
%
%
%   Customization Tips:
%      *The structure which is input, POLYOVRAPRX, can be generated by a 
%       custom overaproximation procedure.  The overapproximation must be 
%       of the form specified in eqn (2) of [Bauer et. al. NECSYS, 2012]. 
%       NOTE: TOD analysis requires the state vector to be ordered in a 
%       specific way.
%
%   See also GENPOLYOVRAPRX
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    warning('off','YALMIP:strict') 
    
    if ~isfield(polyOvrAprx,'Ab')
        error('genPolyOvrAprx.m: Ab is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'Bb')
        error('genPolyOvrAprx.m: Bb is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'Cb')
        error('genPolyOvrAprx.m: Cb is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'Db')
        error('genPolyOvrAprx.m: Db is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'Eb')
        error('genPolyOvrAprx.m: Eb is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'Jb')
        error('genPolyOvrAprx.m: Jb is required to be a field of the input variable POLYOVRAPRX.')
    elseif ~isfield(polyOvrAprx,'JordanBlks')
        error('genPolyOvrAprx.m: JordanBlks is required to be a field of the input variable POLYOVRAPRX.')
    end    
    
    disp(' ')
    disp('STABILITY DATA:')

    
    if strcmp(analysisType,'RR')
        [stable, StabInfo, LMI] = LMI_RR(polyOvrAprx,quantVars);
    elseif strcmp(analysisType,'TOD')
        [stable, StabInfo, LMI] = LMI_TOD(polyOvrAprx,GammaMatrix,quantVars);
    elseif strcmp(analysisType,'OneNode_pardep') || strcmp(analysisType,'OneNode_common')
        [stable, StabInfo, LMI] = LMI_OneNode(polyOvrAprx,analysisType);
    else
        error('checkLMIs.m: Cannot analyze stability. Protocol specified is unknown')
    end
    
end

function [stable, StabInfo, LMI, STABoutput] = LMI_RR(polyOvrAprx,quantVars)

    expectedLMIs = size(polyOvrAprx.Ab,1)*size(polyOvrAprx.Ab,2);
    disp(['  Number of LMIs needed to verify stability = ' num2str(expectedLMIs)])

    Ab = polyOvrAprx.Ab;
    Bb = polyOvrAprx.Bb;
    Cb = polyOvrAprx.Cb;
    Eb = polyOvrAprx.Eb;
    Jb = polyOvrAprx.Jb;
    totalGridPts = size(Ab,2);
    NrNodes = size(Ab,1);

    JJ = polyOvrAprx.JordanBlks;
    
    if isempty(quantVars)==1
        quant='none';
    else
        quant     = quantVars.type;      % determine if log or uniform
        alpha3    = quantVars.alpha3;    % used for both log and uniform
        alpha4    = quantVars.alpha4;    % used for log
        sigma     = quantVars.sigma;     % used for both log and uniform
        omegaMIN  = quantVars.omegaMIN;  % used for uniform         
        omegaMAX  = quantVars.omegaMAX;  % used for uniform
        minimizeIss  = quantVars.minimizeIss;  % used for uniform
    end
    

    switch quant
        %--------------------------------------------------------------------------
        % Not include quantization    
        case{'none'}

                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);

                LmiSet = [];

                % Construct symmetric P matrices
                P = cell(0);
                for ell = 1:NrNodes
                    P{ell} = sdpvar(na);
                end

                % Round-Robin protocol (for one node, equal to common quadratic LF)
                P{NrNodes+1} = P{1};

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
                
                %without dropouts considered
                for ell = 1:NrNodes             % number of nodes
                    for m = 1:totalGridPts     % number of gridpoints

                        % create matrix R, where each ri has the size of the corresponding Jordan block
                        for q = 1:length(JJ{ell})    
                            r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                            r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                            r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        end
                        R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                        LmiSet = LmiSet + set([P{ell}             zeros(na,mb)       Ab{ell,m}'*P{ell+1}  Cb{ell}'*R{ell,m}
                                               zeros(mb,na)       R{ell,m}           Bb{ell}'*P{ell+1}    zeros(mb,mb) 
                                               P{ell+1}*Ab{ell,m} P{ell+1}*Bb{ell}   P{ell+1}             zeros(na,mb)
                                               R{ell,m}*Cb{ell}   zeros(mb,mb)       zeros(mb,na)         R{ell,m}         ]>0);
                        
                       % LmiSet = LmiSet + set([P{ell}               Ab{ell,m}'*P{ell+1};                                               
                       %                        P{ell+1}*Ab{ell,m}   P{ell+1}]>0);
                                              
                                           
                    end
                end  
                
                LMI = LmiSet;
                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                pres      = checkset(LMI);
                inFeas = StabInfo.problem;

                %- Check feasibility
                stable = 1;

                % first check - problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - residuals
                PrimalRes = pres > 0;
                isPos     = find(PrimalRes==0,1);
                if isempty(isPos)==0
                    stable = 0;
                end   

        % ##########################################################################
        %--------------------------------------------------------------------------
        % Include quantization
        case{'log'}

                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);
                za = size(Eb{1},2);         

                LmiSet = []; 
                P = cell(0);
                for ell = 1:NrNodes
                    P{ell} = sdpvar(na);
                end

                P{NrNodes+1} = P{1};

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
                
                %without drops considered
                for ell = 1:NrNodes                   % number of nodes
                    for m = 1:totalGridPts     % number of gridpoints

                        % create matrix R, where each ri has the size of the corresponding Jordan block
                        for q = 1:length(JJ{ell})
                            r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                            r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                            r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        end
                        R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                        LmiSet = LmiSet + set([P{ell}       zeros(na,za)        zeros(na,mb)        Ab{ell,m}'*P{ell+1} Cb{ell}'*R{ell,m}   alpha3*eye(na,na) alpha4*Ht'
                                       zeros(za,na)         sigma*eye(za,za)    zeros(za,mb)        Eb{ell,m}'*P{ell+1} Jb{ell}'*R{ell,m}   zeros(za,na)      zeros(za,za)
                                       zeros(mb,na)         zeros(mb,za)        R{ell,m}            Bb{ell}'*P{ell+1}  zeros(mb,mb)        zeros(mb,na)      zeros(mb,za)
                                       P{ell+1}*Ab{ell,m}   P{ell+1}*Eb{ell,m}  P{ell+1}*Bb{ell}   P{ell+1}            zeros(na,mb)        zeros(na,na)      zeros(na,za)
                                       R{ell,m}*Cb{ell}     R{ell,m}*Jb{ell}    zeros(mb,mb)        zeros(mb,na)        R{ell,m}            zeros(mb,na)      zeros(mb,za)
                                       alpha3*eye(na,na)    zeros(na,za)        zeros(na,mb)        zeros(na,na)        zeros(na,mb)        alpha3*eye(na,na) zeros(na,za)
                                       alpha4*Ht            zeros(za,za)        zeros(za,mb)        zeros(za,na)        zeros(za,mb)        zeros(za,na)      alpha4*eye(za,za) ] > 0);  
                    end
                end
                
                LMI = LmiSet;
                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                pres      = checkset(LMI);
                inFeas = StabInfo.problem;

                Pp = cell(0);  
                for ell = 1:NrNodes
                    Pp{ell} = double(P{ell});
                end

                %- Check feasibility
                stable = 1;

                % first check - problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - residuals
                PrimalRes = pres > 0;
                isPos     = find(PrimalRes==0,1);
                if isempty(isPos)==0
                    stable = 0;
                end

                STABoutput.P = Pp;


        % ##########################################################################
        % -------------------------------------------------------------------------
        % Uniform quantizer (UB computation)
        case{'uniform'}

                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);
                za = size(Eb{1},2);         

                LmiSet = []; LmiSeteq = [];
                P = cell(0);
                for ell = 1:NrNodes
                    P{ell} = sdpvar(na);
                    LmiSeteq = LmiSeteq + set(omegaMIN*eye(na)<=P{ell}<=omegaMAX*eye(na));
                end

                P{NrNodes+1} = P{1};

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
                
                %without drops considered
                for ell = 1:NrNodes                   % number of nodes
                    for m = 1:totalGridPts     % number of gridpoints

                        % create matrix R, where each ri has the size of the corresponding Jordan block
                        for q = 1:length(JJ{ell})
                            r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                            r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                            r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        end
                        R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                        LmiSet = LmiSet + set([P{ell}       zeros(na,za)        zeros(na,mb)      Ab{ell,m}'*P{ell+1}   Cb{ell}'*R{ell,m}   alpha3*eye(na,na)
                                       zeros(za,na)         sigma*eye(za,za)    zeros(za,mb)      Eb{ell,m}'*P{ell+1}   Jb{ell}'*R{ell,m}   zeros(za,na)
                                       zeros(mb,na)         zeros(mb,za)        R{ell,m}          Bb{ell}'*P{ell+1}    zeros(mb,mb)        zeros(mb,na)
                                       P{ell+1}*Ab{ell,m}   P{ell+1}*Eb{ell,m}  P{ell+1}*Bb{ell} P{ell+1}              zeros(na,mb)        zeros(na,na)
                                       R{ell,m}*Cb{ell}     R{ell,m}*Jb{ell}    zeros(mb,mb)      zeros(mb,na)          R{ell,m}            zeros(mb,na)
                                       alpha3*eye(na,na)    zeros(na,za)        zeros(na,mb)      zeros(na,na)          zeros(na,mb)        alpha3*eye(na,na)   ]> 0);
                    end
                end 
                
                %-- Constraints
                neq = size(LmiSeteq,1);
                LMI = LmiSet + LmiSeteq;

                %-- Solve LMIs with SeDuMi
                if minimizeIss == 1    % minimize sigma
                    sigma = sdpvar(1);
                    disp('  Solving LMIs...')
                    StabInfo  = solvesdp(LMI,sigma,sdpsettings('solver','sedumi','verbose',0));
                    kappaMin     = double(sigma);
                else
                    disp('  Solving LMIs...')
                    StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                    kappaMin  = sigma;
                end

                pres      = checkset(LMI);
                inFeas = StabInfo.problem;

                checkP = pres(1:end-neq);
                checkE = pres(end-neq+1:end);

                Pp = cell(0);
                eigPmin = zeros(NrNodes,1);
                eigPmax = zeros(NrNodes,1);
                for ell = 1:NrNodes
                    Pp{ell} = double(P{ell});
                    eigPmin(ell,1) = min(eig(Pp{ell}));
                    eigPmax(ell,1) = max(eig(Pp{ell}));
                end
                MinEigP = min(eigPmin);
                MaxEigP = max(eigPmax);

                % ISS-gain
                gammaBar = sqrt((MaxEigP*kappaMin)/(MinEigP*alpha3));

                % first check - infeasible problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - Positive definiteness P
                checkPosDef = MinEigP > 1e-14;

                if checkPosDef==0
                    stable = 0;
                    disp('P matrices not positive definite')
                end 

                % third check - P residuals
                PrimalRes = checkP > 0;
                isPos     = find(PrimalRes==0);
                if isempty(isPos)==0
                    stable = 0;
                    disp('Residuals not satisfied')
                end    

                % fourth check - Equality residuals
                EqualRes = abs(checkE) < 1e-8; % small enough
                isPosE   = find(EqualRes==0,1);
                if isempty(isPosE)==0
                    if isempty(isPos)==1
                    stable = 1;
                    else
                        stable = 0;
                        disp('Equality residuals not satisfied')
                    end
                end 

                % output
                STABoutput.P = Pp;
                STABoutput.gammaBar = gammaBar;
                STABoutput.kappa = kappa;

    end
end

function [stable, StabInfo, LMI, STABoutput] = LMI_TOD(polyOvrAprx,GammaMatrix,quantVars)

    expectedLMIs = size(polyOvrAprx.Ab,1)*size(polyOvrAprx.Ab,2)+1;
    disp(['  Number of LMIs needed to verify stability = ' num2str(expectedLMIs)])
    
    Ab = polyOvrAprx.Ab;
    Bb = polyOvrAprx.Bb;
    Cb = polyOvrAprx.Cb;
    Eb = polyOvrAprx.Eb;
    Jb = polyOvrAprx.Jb;
    JJ = polyOvrAprx.JordanBlks;

    NrNodes = size(Ab,1);
    totalGridPts = size(Ab,2);

    if isempty(quantVars)==1
        quant='none';
    else
        quant     = quantVars.type;      % determiune if log or uniform
        alpha3    = quantVars.alpha3;    % used for both log and uniform
        alpha4    = quantVars.alpha4;    % used for log
        sigma     = quantVars.sigma;     % used for both log and uniform
        omegaMIN  = quantVars.omegaMIN;  % used for uniform         
        omegaMAX  = quantVars.omegaMAX;  % used for uniform
        %minimizeIss  = quantVars.minimizeIss; % not used for TOD
    end


    switch quant
        %--------------------------------------------------------------------------
        % Not include quantization    
        case{'none'}

                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);

                LmiSet  = []; LmiSeteq = [];
                P   = sdpvar(na);
                pi  = sdpvar(NrNodes,NrNodes,'full');
                LmiSet  = LmiSet + set(pi(:)>0);

                V = cell(0);
                for ell = 1:NrNodes
                    V{ell} = blkdiag(zeros((na-size(GammaMatrix{ell},1))),-GammaMatrix{ell});
                    LmiSeteq  = LmiSeteq+set(sum(pi(:,ell))==1); 
                end

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); piV = cell(0);
                
                %without drops considered
                for ell = 1:NrNodes                   % number of nodes
                    for m = 1:totalGridPts     % number of gridpoints
                        % create matrix R, where each ri has the size of the corresponding Jordan block
                        for q = 1:length(JJ{ell})
                            r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                            r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                            r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        end
                        R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                        Pi = P + V{ell};

                        sumpiV = zeros(max(size(V{1})));
                        for qq = 1:NrNodes 
                            piV{qq} = pi(qq,ell)*V{qq};
                            sumpiV = sumpiV+piV{qq};
                        end
                        Pj = P+sumpiV;

                        LmiSet = LmiSet + set([ Pi               zeros(na,mb)  Ab{ell,m}'*Pj     Cb{ell}'*R{ell,m}
                                                zeros(mb,na)     R{ell,m}      Bb{ell}'*Pj      zeros(mb,mb) 
                                                Pj*Ab{ell,m}     Pj*Bb{ell}   Pj                zeros(na,mb)
                                                R{ell,m}*Cb{ell} zeros(mb,mb)  zeros(mb,na)      R{ell,m}         ]>0);
                    end
                end
                
                LMI = LmiSet+LmiSeteq;

                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));

                inFeas = StabInfo.problem;
                pres      = checkset(LMI);

                checkP = pres(1:end-NrNodes);
                checkE = pres(end-NrNodes+1:end);

                %- Check feasibility
                stable = 1;

                % first check - infeasible problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - P residuals
                PrimalRes = checkP > 0;
                isPos     = find(PrimalRes==0,1);
                if isempty(isPos)==0
                    stable = 0;
                end    

                % third check - E residuals
                EqualRes = abs(checkE) < 1e-8; % deviation of sum(pi) of 1e-8 allowed
                isPosE   = find(EqualRes==0,1);
                if isempty(isPosE)==0
                    stable = 0;
                    disp('sum(pi)~=1')
                end 

        % ##########################################################################
        %--------------------------------------------------------------------------
        % Include quantization
        case{'log'}

                %----------------------------------------------------------------------
                %-- Only Relaxation at m
                %----------------------------------------------------------------------

                %-- Create Matrix Qi --------------------------------------------------
                % Qi = [0  0  |  0        = [Qi11 | Qi12
                %       0 -Ti |  Ti          -----|----- 
                %       - - - | - -          Qi21 | Qi22]     
                %       0  Ti | -Ti]      Where Qi21 = Qi12^T
                % 
                % Sizes:
                % Qi = [(np+nc) x (np+nc) (np+nc) x (np+nc) | (np+nc) x NrNodes        
                %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes           
                %       - - - - - - - - - - - - - - - - - - | - - - - - - - - -               
                %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes]      
                %----------------------------------------------------------------------
                q11 = cell(0); q12 = cell(0); q22 = cell(0);
                for ell = 1:NrNodes
                    q11{ell} = [zeros((np+nc),(np+nc)) zeros((np+nc),NrNodes)
                               zeros(NrNodes,(np+nc)) -GammaMatrix{ell}];
                    q12{ell} = [zeros((np+nc),NrNodes) 
                               GammaMatrix{ell}];
                    q22{ell} = -GammaMatrix{ell};
                end

                %-- create \hat{Q}^i and \hat{Q}^j ------------------------------------
                LmiSeteq = [];
                xi = cell(0); qi11 = cell(0); qi12 = cell(0); qi22 = cell(0);
                for ell = 1:NrNodes
                    for ff = 1:NrNodes
                        if ell == ff
                            % relaxation k
                            xi{ell,ff} = 0;
                            qi11{ell,ff} = xi{ell,ff}*(q11{ell}-q11{ff});
                            qi12{ell,ff} = xi{ell,ff}*(q12{ell}-q12{ff});
                            qi22{ell,ff} = xi{ell,ff}*(q22{ell}-q22{ff});
                        else
                            % relaxation k
                            xi{ell,ff} = sdpvar(1,1);
                            qi11{ell,ff} = xi{ell,ff}*(q11{ell}-q11{ff});
                            qi12{ell,ff} = xi{ell,ff}*(q12{ell}-q12{ff});
                            qi22{ell,ff} = xi{ell,ff}*(q22{ell}-q22{ff});
                            LmiSeteq = LmiSeteq + set(xi{ell,ff}>=0);
                        end
                    end
                end

                % Add elements together
                Qhi11 = cell(0); Qhi12 = cell(0); Qhi22 = cell(0);
                for ell = 1:NrNodes
                    % relaxation k
                    tmpi11 = zeros((np+nc+NrNodes),(np+nc+NrNodes));
                    tmpi12 = zeros((np+nc+NrNodes),NrNodes);
                    tmpi22 = zeros(NrNodes,NrNodes);
                    for ff = 1:NrNodes
                        % relaxation k
                        tmpi11 = tmpi11+qi11{ell,ff};
                        tmpi12 = tmpi12+qi12{ell,ff};
                        tmpi22 = tmpi22+qi22{ell,ff};
                    end
                    % relaxation k
                    Qhi11{ell,1} = tmpi11;
                    Qhi12{ell,1} = tmpi12;
                    Qhi22{ell,1} = tmpi22;
                end
                %----------------------------------------------------------------------    

                %----------------------------------------------------------------------
                % Actual Stability analysis
                
                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);
                za = size(Eb{1},2); 

                LmiSet = [];
                P = cell(0);
                for ell = 1:NrNodes
                    P{ell} = sdpvar(na); 
                end

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0);
                
                %without drops considered
                for ell = 1:NrNodes                  % number of nodes
                    for m = 1:totalGridPts    % number of gridpoints
                    % create matrix R, where each ri has the size of the corresponding Jordan block
                    for q = 1:length(JJ{ell})    % length(JJ{ell}) = number of Jordan blocks, JJ = size Jordan block
                        r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                        r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                    end
                    R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                    for jj = 1:NrNodes   % number of nodes
                        LmiSet = LmiSet + set([P{ell}+Qhi11{ell}    Qhi12{ell}                  zeros(na,mb)    Ab{ell,m}'*P{jj} Cb{ell}'*R{ell,m}  alpha3*eye(na,na) alpha4*Ht'
                                                Qhi12{ell}'         sigma*eye(za,za)+Qhi22{ell} zeros(za,mb)    Eb{ell,m}'*P{jj} Jb{ell}'*R{ell,m}  zeros(za,na)      zeros(za,za)
                                                zeros(mb,na)        zeros(mb,za)                R{ell,m}        Bb{ell}'*P{jj}  zeros(mb,mb)       zeros(mb,na)      zeros(mb,za)
                                                P{jj}*Ab{ell,m}     P{jj}*Eb{ell,m}             P{jj}*Bb{ell}  P{jj}            zeros(na,mb)       zeros(na,na)      zeros(na,za)
                                                R{ell,m}*Cb{ell}    R{ell,m}*Jb{ell}            zeros(mb,mb)    zeros(mb,na)     R{ell,m}           zeros(mb,na)      zeros(mb,za)
                                                alpha3*eye(na,na)   zeros(na,za)                zeros(na,mb)    zeros(na,na)     zeros(na,mb)       alpha3*eye(na,na) zeros(na,za)
                                                alpha4*Ht           zeros(za,za)                zeros(za,mb)    zeros(za,na)     zeros(za,mb)       zeros(za,na)      alpha4*eye(za,za) ] > 0);
                    end % end for jj
                    end % end for gridpnt
                end % end for NrNodes 
                
                
                neq = size(LmiSeteq,1);
                LMI = LmiSet + LmiSeteq;

                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                pres = checkset(LMI);
                inFeas = StabInfo.problem;

                checkP = pres(1:end-neq);
                checkE = pres(end-neq+1:end);

                checkxi = zeros(NrNodes,NrNodes); xiout = cell(0); 
                for ell = 1:NrNodes
                    P{ell} = double(P{ell});
                    for jj = 1:NrNodes
                        xiout{ell,jj} = double(xi{ell,jj});
                        checkxi(ell,jj) = xiout{ell,jj} >= 0;
                    end
                end

                %- Check feasibility
                stable = 1;

                % first check - infeasible problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - Positive definiteness P
                minEigP = zeros(NrNodes,1);
                for ell=1:NrNodes
                    minEigP(ell,1)=min(eig(P{ell}));
                end

                checkPosDef = find(minEigP < 1e-14,1);

                if isempty(checkPosDef)==0
                    stable = 0;
                    disp('P matrices not positive definite')
                end 

                % third check - P residuals
                PrimalRes = checkP > 0;
                isPos     = find(PrimalRes==0);
                if isempty(isPos)==0
                    stable = 0;
                    disp('Residuals not satisfied')
                end    

                % fourth check - Equality residuals
                EqualRes = abs(checkE) < 1e-8; % small enough
                isPosE   = find(EqualRes==0,1);
                isPosSDxi= find(checkxi==0,1);
                if isempty(isPosE)==0
                    if isempty(isPosSDxi)==1 && isempty(isPos)==1
                    stable = 1;
                    else
                        stable = 0;
                        disp('Equality residuals not satisfied')
                    end
                end 

        % -------------------------------------------------------------------------
        % Uniform quantizer (UB computation)

        case{'uniform'}

            %----------------------------------------------------------------------
                % Quantization included and TOD protocol
                case{'TOD'}

                %----------------------------------------------------------------------
                %-- Only Relaxation at k
                %----------------------------------------------------------------------

                %-- Create Matrix Qi --------------------------------------------------
                % Qi = [0  0  |  0        = [Qi11 | Qi12
                %       0 -Ti |  Ti          -----|----- 
                %       - - - | - -          Qi21 | Qi22]     
                %       0  Ti | -Ti]      Where Qi21 = Qi12^T
                % 
                % Sizes:
                % Qi = [(np+nc) x (np+nc) (np+nc) x (np+nc) | (np+nc) x NrNodes        
                %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes           
                %       - - - - - - - - - - - - - - - - - - | - - - - - - - - -               
                %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes]      
                %----------------------------------------------------------------------
                q11 = cell(0); q12 = cell(0); q22 = cell(0);
                for ell = 1:NrNodes
                    q11{ell} = [zeros((np+nc),(np+nc)) zeros((np+nc),NrNodes)
                               zeros(NrNodes,(np+nc)) -GammaMatrix{ell}];
                    q12{ell} = [zeros((np+nc),NrNodes) 
                               GammaMatrix{ell}];
                    q22{ell} = -GammaMatrix{ell};
                end

                %-- create \hat{Q}^i and \hat{Q}^j ------------------------------------
                LmiSeteq = [];
                xi = cell(0); qi11 = cell(0); qi12 = cell(0); qi22 = cell(0);
                for ell = 1:NrNodes
                    for ff = 1:NrNodes
                        if ell == ff
                            % relaxation k
                            xi{ell,ff} = 0;
                            qi11{ell,ff} = xi{ell,ff}*(q11{ell}-q11{ff});
                            qi12{ell,ff} = xi{ell,ff}*(q12{ell}-q12{ff});
                            qi22{ell,ff} = xi{ell,ff}*(q22{ell}-q22{ff});
                        else
                            % relaxation k
                            xi{ell,ff} = sdpvar(1,1);
                            qi11{ell,ff} = xi{ell,ff}*(q11{ell}-q11{ff});
                            qi12{ell,ff} = xi{ell,ff}*(q12{ell}-q12{ff});
                            qi22{ell,ff} = xi{ell,ff}*(q22{ell}-q22{ff});
                            LmiSeteq = LmiSeteq + set(xi{ell,ff}>=0);
                        end
                    end
                end

                % Add elements together
                Qhi11 = cell(0); Qhi12 = cell(0); Qhi22 = cell(0);
                for ell = 1:NrNodes
                    % relaxation k
                    tmpi11 = zeros((np+nc+NrNodes),(np+nc+NrNodes));
                    tmpi12 = zeros((np+nc+NrNodes),NrNodes);
                    tmpi22 = zeros(NrNodes,NrNodes);
                    for ff = 1:NrNodes
                        % relaxation k
                        tmpi11 = tmpi11+qi11{ell,ff};
                        tmpi12 = tmpi12+qi12{ell,ff};
                        tmpi22 = tmpi22+qi22{ell,ff};
                    end
                    % relaxation k
                    Qhi11{ell,1} = tmpi11;
                    Qhi12{ell,1} = tmpi12;
                    Qhi22{ell,1} = tmpi22;
                end
                %----------------------------------------------------------------------    

                %----------------------------------------------------------------------
                % Actual Stability analysis
                
                % matrix sizes
                % must be constant over all nodes!
                na = size(Ab{1,1},1);
                mb = size(Bb{1},2);
                za = size(Eb{1},2); 

                LmiSet = [];
                P = cell(0);
                for ell = 1:NrNodes
                    P{ell} = sdpvar(na);
                    LmiSeteq = LmiSeteq + set(omegaMIN*eye(na)<=P{ell}<=omegaMAX*eye(na));
                end

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0);
                
                %without drops considered
                for ell = 1:NrNodes                  % number of nodes
                    for m = 1:totalGridPts    % number of gridpoints
                    % create matrix R, where each ri has the size of the corresponding Jordan block
                    for q = 1:length(JJ{ell})
                        r1{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                        r2{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        r3{ell,m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                    end
                    R{ell,m} = blkdiag(blkdiag(r1{ell,m,:}),blkdiag(r2{ell,m,:}),blkdiag(r3{ell,m,:}));

                    for jj = 1:NrNodes   % number of nodes
                        LmiSet = LmiSet + set([P{ell}+Qhi11{ell}    Qhi12{ell}                  zeros(na,mb)    Ab{ell,m}'*P{jj} Cb{ell}'*R{ell,m}  alpha3*eye(na,na) 
                                                Qhi12{ell}'         sigma*eye(za,za)+Qhi22{ell} zeros(za,mb)    Eb{ell,m}'*P{jj} Jb{ell}'*R{ell,m}  zeros(za,na)      
                                                zeros(mb,na)        zeros(mb,za)                R{ell,m}        Bb{ell}'*P{jj}  zeros(mb,mb)       zeros(mb,na)      
                                                P{jj}*Ab{ell,m}     P{jj}*Eb{ell,m}             P{jj}*Bb{ell}  P{jj}            zeros(na,mb)       zeros(na,na)      
                                                R{ell,m}*Cb{ell}    R{ell,m}*Jb{ell}            zeros(mb,mb)    zeros(mb,na)     R{ell,m}           zeros(mb,na)      
                                                alpha3*eye(na,na)   zeros(na,za)                zeros(na,mb)    zeros(na,na)     zeros(na,mb)       alpha3*eye(na,na)] > 0);
                    end % end for jj
                    end % end for gridpnt
                end % end for NrNodes 
                
                neq = size(LmiSeteq,1);
                LMI = LmiSet + LmiSeteq;

                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                pres = checkset(LMI);
                inFeas = StabInfo.problem;
                kappa = sigma;

                checkP = pres(1:end-neq);
                checkE = pres(end-neq+1:end);

                checkxi = zeros(NrNodes,NrNodes); xiout = cell(0); 
                for ell = 1:NrNodes
                    P{ell} = double(P{ell});
                    for jj = 1:NrNodes
                        xiout{ell,jj} = double(xi{ell,jj});
                        checkxi(ell,jj) = xiout{ell,jj} >= 0;
                    end
                end

                Pp = cell(0);
                eigPmin = zeros(NrNodes,1);
                eigPmax = zeros(NrNodes,1);
                for ell = 1:NrNodes
                    Pp{ell} = double(P{ell});
                    eigPmin(ell,1) = min(eig(Pp{ell}));
                    eigPmax(ell,1) = max(eig(Pp{ell}));
                end
                MinEigP = min(eigPmin);
                MaxEigP = max(eigPmax);

                % ISS-gain
                gammaBar = sqrt((MaxEigP*kappa)/(MinEigP*alpha3));

                %- Check feasibility
                stable = 1;

                % first check - infeasible problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - Positive definiteness P
                minEigP = zeros(NrNodes,1);
                for ell=1:NrNodes
                    minEigP(ell,1)=min(eig(P{ell}));
                end

                checkPosDef = find(minEigP < 1e-14,1);

                if isempty(checkPosDef)==0
                    stable = 0;
                    disp('P matrices not positive definite')
                end 

                % third check - P residuals
                PrimalRes = checkP > 0;
                isPos     = find(PrimalRes==0);
                if isempty(isPos)==0
                    stable = 0;
                    disp('Residuals not satisfied')
                end    

                % fourth check - Equality residuals
                EqualRes = abs(checkE) < 1e-8; % small enough
                isPosE   = find(EqualRes==0,1);
                isPosSDxi= find(checkxi==0,1);
                if isempty(isPosE)==0
                    if isempty(isPosSDxi)==1 && isempty(isPos)==1
                    stable = 1;
                    else
                        stable = 0;
                        disp('Equality residuals not satisfied')
                    end
                end 

                % output
                STABoutput.P = Pp;
                STABoutput.gammaBar = gammaBar;
                STABoutput.kappa = kappa;

    end % end switch quant
end

function [stable, StabInfo, LMI] = LMI_OneNode(polyOvrAprx,lyap)


    Ab = polyOvrAprx.Ab;
    Bb = polyOvrAprx.Bb;
    Cb = polyOvrAprx.Cb;
    %Eb = polyOvrAprx.Eb;  % not used, quantization not implemented
    %Jb = polyOvrAprx.Jb;  % not used, quantization not implemented
    JJ = polyOvrAprx.JordanBlks;
    
    totalGridPts = size(Ab,2);


    switch lyap
                % ---------------------------------------------------------------------
                % no communication constraints: parameter depended quadratic Lyapunov 
                % function
            case{'OneNode_pardep'}

                    % NOT FULmiSetY ROBUST!!!!!!
                    % - does not include quantization
                    % - should only be used for the examples of the MSc Thesis of J.J.C. 
                    %   van Schendel

                % matrix sizes
                na = size(Ab{1},1);
                mb = size(Bb{1},2);

                LmiSet = [];

                % Construct symmetric P matrix
                P = cell(0);
                for m = 1:totalGridPts     % number of gridpoints
                    P{m} = sdpvar(na);
                end

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
                for m = 1:totalGridPts     % number of gridpoints

                    % create matrix R, where each ri has the size of the corresponding Jordan block
                    for q = 1:length(JJ{ell})
                        r1{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                        r2{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        r3{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                    end
                    R{m} = blkdiag(blkdiag(r1{m,:}),blkdiag(r2{m,:}),blkdiag(r3{m,:}));

                    for z=1:totalGridPts     % number of gridpoints
                    LmiSet = LmiSet + set([ P{m}         zeros(na,mb)   Ab{m}'*P{z}    Cb{1}'*R{m}
                                            zeros(mb,na) R{m}           Bb{ell}'*P{z} zeros(mb,mb) 
                                            P{z}*Ab{m}   P{z}*Bb{ell}  P{z}           zeros(na,mb)
                                            R{m}*Cb{1}   zeros(mb,mb)   zeros(mb,na)   R{m}         ]>0);
                    end
                end
                LMI=LmiSet;
                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                check     = checklmi(LMI);
                inFeas = StabInfo.problem;

                for m = 1:totalGridPts
                    P{m} = double(P{m});
                    R{m} = double(R{m});
                end

                %- Check feasibility
                stable = 1;

                % first check - problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - residuals
                PrimalRes = check > 0;
                isPos     = find(PrimalRes==0,1);
                if isempty(isPos)==0
                    stable = 0;
                end

            % ---------------------------------------------------------------------
            % no communication constraints: common quadratic Lyapunov function    
            case{'OneNode_common'}

                    % NOT FULmiSetY ROBUST!!!!!!
                    % should only be used for the examples of the MSc Thesis of J.J.C. 
                    % van Schendel

                % matrix sizes
                na = size(Ab{1},1);
                mb = size(Bb{1},2);    

                LmiSet = [];

                % Construct symmetric P matrix
                P = sdpvar(na);

                r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
                for m = 1:totalGridPts     % number of gridpoints

                    % create matrix R, where each ri has the size of the corresponding Jordan block
                    for q = 1:length(JJ{ell})
                        r1{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));   
                        r2{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                        r3{m,q} = sdpvar(1,1)*eye(max(size(JJ{ell}{q})));
                    end
                    R{m} = blkdiag(blkdiag(r1{m,:}),blkdiag(r2{m,:}),blkdiag(r3{m,:}));

                    LmiSet = LmiSet + set([ P            zeros(na,mb)   Ab{m}'*P       Cb{1}'*R{m}
                                            zeros(mb,na) R{m}           Bb{ell}'*P    zeros(mb,mb) 
                                            P*Ab{m}      P*Bb{ell}     P              zeros(na,mb)
                                            R{m}*Cb{1}   zeros(mb,mb)   zeros(mb,na)   R{m}         ]>0);
                end

                LMI = LmiSet;
                %-- Solve LMIs with SeDuMi
                disp('  Solving LMIs...')
                StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
                check     = checklmi(LMI);
                inFeas = StabInfo.problem;

                for m = 1:totalGridPts
                    R{m} = double(R{m});
                end

                %- Check feasibility
                stable = 1;

                % first check - problem   
                if inFeas == 1
                    stable = 0;
                end

                % second check - residuals
                PrimalRes = check > 0;
                isPos     = find(PrimalRes==0,1);
                if isempty(isPos)==0
                    stable = 0;
                end    

    end % end switch protocol
end