function [isstable, StabInfo, LMI] = analyzeStochStability(ncs)
%ANALYZESTOCHSTABILITY  Verifies LMI conditions to determine if a NCS model 
%                       with probability distribution is mean-square stable.
%
%   [STABLE,STABINFO,LMI] = ANALYZESTOCHSTABILITY(NCS)
%   Verifies LMI conditions to determine if mean-square stability STABLE 
%   can be guaranteed of a NCS model. For more information, see
%   [Antunes et. al., American Control Conference 2011]
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    protocol = ncs.getNetwork.protocol;
    
    if(isempty(ncs.getNetwork.timingDist))
        error('The property timingDist must be set before stochastic stability can be analyzed');
    end
    
    warning('off','YALMIP:strict')
     
    
    disp(' ')
    disp('STABILITY DATA:')

     
    if strcmp(protocol,'RR')

        dist = ncs.getNetwork.timingDist;       
        n_ms = size(dist,1);                       % Number of network measurements
        nrNodes = length(ncs.getNetwork.nodesU);   % Number of sensors (i.e. number of systems)

        expectedLMIs = nrNodes;
        disp(['  Number of LMIs needed to verify stability = ' num2str(expectedLMIs)]);    
    
        A=cell(nrNodes,n_ms);
        prob = cell(n_ms,1);
        P = cell(nrNodes,1);

        for sigma = 1:nrNodes
            for j = 1:n_ms
                A{sigma,j} = ncs.evaluateDtNcsModel(dist(j,1),dist(j,2),sigma);
                prob{j} = dist(j,3)/100;
            end
            P{sigma}=sdpvar(size(A{sigma,1},1));
        end

        F = cell(nrNodes,1);
        for sigma=1:nrNodes
            F{sigma}=zeros(size(A{sigma,1},1));
        end
        
        for j = 1:n_ms
            for sigma=1:nrNodes

                if sigma==nrNodes
                   sigmaPlus = 1;
                else
                   sigmaPlus = sigma+1;
                end    
                 F{sigma} =  F{sigma}+prob{j}.*(A{sigma,j}'*P{sigmaPlus}*A{sigma,j});
            end
        end

        LMI = [];
        for sigma=1:nrNodes
            LMI = LMI + set(P{sigma}>0);
            LMI = LMI + set(F{sigma}-P{sigma}<0);          
        end
        
        %-- Solve LMIs with SeDuMi
        disp('  Solving LMIs...')
        StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
        pres      = checkset(LMI);
        inFeas    = StabInfo.problem;

        %checkset(LMI);

        %- Check feasibility
        isstable = 1;

        % first check - problem
        if inFeas == 1
            isstable = 0;
        end

        % second check - residuals
        PrimalRes = pres > 0;
        isPos     = find(PrimalRes==0,1);
        if isempty(isPos)==0
            isstable = 0;
        end

    elseif strcmp(protocol,'TOD')
        error('Stochastic analysis of the TOD protocol has not been implemented yet')
    end
        
end