function [dtNcsMdl,GammaMatrix] = genDtNcsModel(ncs,verbose)
% GENDTNCSMODEL From the properties of an NCS class object, generates the 
%               matrices A{l},B{l},C{l},D{l},E{l},J{l} and Atilde{l}, l=1,...,N,  
%               which describe the following discrete-time model:
%
%               x_{k+1} = (A{l} + B{l}* Phi_{k,l} *C{l})x_{k} +
%                           (E{l} + D{l}* Phi_{k,l} *J{l})w_{k}
%
%               where x_k is the value of the NCS state vector at time k, w_k 
%               is a disturbance and the network uncertainty has the form
%               Phi_{k,l}=blkdiag(A_{hk,l},E_{hk,l},E_{hk-tauk,l}).  Atilde{l} 
%               is used for defining the matrices Phi_{k,l}. See 
%               [Bauer et. al. NECSYS, 2012]  eqn (1) for more info.
%
%   [DTNCSMDL,GAMMAMATRIX] = GENDTNCSMODEL(NCS) 
%   Extracts the information in an NCS object and generates the matrices 
%   A{l},B{l},C{l},D{l},E{l},J{l} and stores them in DTNCSMDL.  
%
%   INPUTS
%   ------------------------------------------------------
%       NCS         - NCS object created with the ncsEditor
%
%   OUTPUTS
%   ------------------------------------------------------
%       DTNCSMDL    - strucuture containing information about 
%                     the discrete-time NCS model
%                     ~~~~Fields~~~~
%                     DTNCSMDL.A       - A matrix assoc. with model 
%                     DTNCSMDL.B       - B matrix assoc. with model
%                     DTNCSMDL.C       - C matrix assoc. with model
%                     DTNCSMDL.D       - D matrix assoc. with model
%                     DTNCSMDL.E       - E matrix assoc. with model
%                     DTNCSMDL.J       - J matrix assoc. with model
%                     DTNCSMDL.Atilde  - matrix assoc. with uncertainty
%                     DTNCSMDL.tPoly   - refinement of uncertain region
%                     ~~~~~~~~~~~~~~~~~~
%       GAMMAMATRIX - structure containing protocol information. Needed
%                     for TOD protocol analysis in analyzePolyOvrAprx.m.
%
%   Customization Tip:
%     *The structure which is output, DTNCSMDL, can be used in custom
%      overaproximation techniques.
%
%   See also NCS
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

%%
% setup1 := NCS model with continuous-time controller where both y and u are sent over a network;
% setup2 := NCS model with discrete-time controller where both y and u are sent over a network;
% setup3 := NCS model with continuous-time controller where only y is transmitted over a network 
%           (assume that the controller communicates its values continuously and without delays);
% setup4 := NCS model with state-feedback 
% setup5 := discrete-time observer-based feedback

%% Error Checking

    if ~isobject(ncs)
        error('genDtNcsModel.m: Input parameter NCS must be an object of type ncs.  Use the ncsEditor to create and edit NCS objects.')
    end

%% Display System Information
    
    if nargin==1
        verbose=0;
    end
    
    net=ncs.getNetwork;
    if verbose==1

        disp('SYSTEM DATA:')

        if net.txIntvls(1)-net.txIntvls(2) == 0
            disp(['  txIntvl    =  ' num2str(net.txIntvls(1))])
        else
            disp(['  txIntvls   = [' num2str(net.txIntvls(1)) ',  ' num2str(net.txIntvls(2)) ']'])
        end
        if net.delays(1) - net.delays(2) == 0
            disp(['  delay      =  ' num2str(net.delays(1))]) 
        else
            disp(['  delays     = [' num2str(net.delays(1)) ',  ' num2str(net.delays(2)) ']']) 
        end 

        % Total number of nodes
        nrNodes = length(net.nodesU);
        disp(['  Number of Nodes:  ' num2str(nrNodes)])
        disp(['  Protocol:         ' net.protocol])
    end
    
%% Determine which setup to use
    c=ncs.getController;
    switch c.ctrlType
        case{'Static Feedback'}
           [dtNcsMdl,GammaMatrix]=setup4(ncs,verbose);
        case{'C-LTI Dynamic Feedback'}
            [dtNcsMdl,GammaMatrix]=setup1(ncs,verbose);
        case{'C-LTI Dynamic Feedback (uWired)'}           
            [dtNcsMdl,GammaMatrix]=setup3(ncs,verbose);
        case{'D-LTI Dynamic Feedback'}
            [dtNcsMdl,GammaMatrix]=setup2(ncs,verbose);
        case{'D-LTI Observer-Based Feedback'}
            [dtNcsMdl,GammaMatrix]=setup5(ncs,verbose);
        otherwise
            error('genDtNcsModel.m: Cannot generate discrete-time NCS model.  Controller type is unknown.')
    end
    
%% Dropouts (modeled as prolongation of the sampling interval) 
    
    % determine polygon with highest allowed drops
    if net.maxDrops
        maxDropPoly=net.tPoly;
        for d=1:net.maxDrops
           maxDropPoly = findNextDropPoly(net.tPoly,maxDropPoly); 
        end

        %create a convex hull around polygon with zero drops and polygon with max drops
        ptsX = [net.tPoly(:,1);maxDropPoly(:,1)];
        ptsY = [net.tPoly(:,2);maxDropPoly(:,2)]; 
        k = convhull(ptsX,ptsY,'simplify',true);
        dtNcsMdl.tPoly = [ptsX(k) ptsY(k)];
    else
        dtNcsMdl.tPoly = net.tPoly;
    end
end

function [dtNcsMdl,GammaMatrix] = setup1(ncs,verbose)
%----------------------------------------------------------------------
% continuous-time controller and both y and u are communicated    
    if verbose==1
        disp('  Controller:           Continuous-time Dynamic Feedback')
        disp('  Channels Transmitted: All inputs and outputs')
    end
    
    net=ncs.getNetwork;
    p=ncs.getPlant;
    c=ncs.getController;
    
    
    GammaMatrix = cell(0);
    nrNodes = length(net.nodesU);
    for jj = 1:nrNodes
        GammaMatrix{jj} = blkdiag(net.nodesY{jj},net.nodesU{jj});
    end
    
    % System Matrices
    Ap = p.A;
    Bp = p.B;
    Cp = p.C;

    Ac=cell(1,nrNodes);
    Bc=cell(1,nrNodes);
    Cc=cell(1,nrNodes);
    Dc=cell(1,nrNodes);

    if length(c.A) == 1
       for ctr=1:nrNodes
           Ac{ctr} = c.A{1};
           Bc{ctr} = c.B{1};
           Cc{ctr} = c.C{1};
           Dc{ctr} = c.D{1};
       end
        dtNcsMdl.Atilde{1} = blkdiag(Ap,Ac{1});   
    else
        Ac=c.A;
        Bc=c.B;
        Cc=c.C;
        Dc=c.D;
        for ctr=1:nrNodes
            dtNcsMdl.Atilde{ctr} = blkdiag(Ap,Ac{ctr});
        end
    end
    
    % Construct matrix Lambda

    % Matrices closed loop system
    for ell = 1:nrNodes

        % Matrix sizes
        [np mp] = size(Bp);
        lp      = size(Cp,1); 
        [nc mc] = size(Bc{ell});
        lc      = size(Cc{ell},1); 

        B = [zeros(np,mc) Bp;Bc{ell} zeros(nc,mp)];
        C = blkdiag(Cp,Cc{ell});
        D = [eye((mc+mp-lc),mc) zeros((mc+mp-lc),(lp+lc-mc));Dc{ell} eye(lc,(lp+lc-mc))];

        
        dtNcsMdl.A{ell} = [zeros(np+nc) zeros(np+nc,mp+lp); C eye(mp+lp,mp+lp)-D\GammaMatrix{ell}];
        dtNcsMdl.E{ell} = [zeros(np+nc,mp+lp);D\GammaMatrix{ell}];
        
        dtNcsMdl.B{ell} = [eye(np+nc);-C];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np+nc);-C]];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np+nc);-C]];

        dtNcsMdl.C{ell} = [eye(np+nc) zeros(np+nc,mp+lp)];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [B*D*C B*D]];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np+nc) -B*GammaMatrix{ell}]];

        dtNcsMdl.D{ell} = dtNcsMdl.B{ell};
        
        dtNcsMdl.J{ell} = zeros((np+nc),mp+lp);
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; zeros((np+nc),mp+lp)];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; B*GammaMatrix{ell}];

    end

end

function [dtNcsMdl,GammaMatrix] = setup2(ncs,verbose)
%----------------------------------------------------------------------
% discrete-time controller and both y and u are communicated     
    if verbose==1
        disp('  Controller:           Discrete-time Dynamic Feedback')
        disp('  Channels Transmitted: All inputs and outputs')
    end
    
    net=ncs.getNetwork;
    p=ncs.getPlant;
    c=ncs.getController;
    
    
    GammaMatrix = cell(0);
    nrNodes = length(net.nodesU);
    for jj = 1:nrNodes
        GammaMatrix{jj} = blkdiag(net.nodesY{jj},net.nodesU{jj});
    end
    
    % System Matrices
    Ap = p.A;
    Bp = p.B;
    Cp = p.C;
    
    Ac=cell(1,nrNodes);
    Bc=cell(1,nrNodes);
    Cc=cell(1,nrNodes);
    Dc=cell(1,nrNodes);
    
    if length(c.A) == 1
       for ctr=1:nrNodes
           Ac{ctr} = c.A{1};
           Bc{ctr} = c.B{1};
           Cc{ctr} = c.C{1};
           Dc{ctr} = c.D{1};
       end
    else
        Ac=c.A;
        Bc=c.B;
        Cc=c.C;
        Dc=c.D;
    end
    dtNcsMdl.Atilde{1} = Ap;

    % Matrix sizes
    [np nu] = size(Bp);
    ny      = size(Cp,1); 
    
    for ell = 1:nrNodes
        nc      = size(Bc{ell},1);

        dtNcsMdl.A{ell} = [zeros(np,np)              zeros(np,nc)                zeros(np,ny)                                                        zeros(np,nu);
                          Bc{ell}*Cp                Ac{ell}                     Bc{ell}*(eye(ny)-net.nodesY{ell})                                   zeros(nc,nu);
                          Cp                        zeros(ny,nc)                eye(ny)-net.nodesY{ell}                                             zeros(ny,nu);
                          -Cc{ell}*Bc{ell}*Cp       Cc{ell}*(eye(nc)-Ac{ell})   Dc{ell}*net.nodesY{ell}-Cc{ell}*Bc{ell}*(eye(ny)-net.nodesY{ell})   eye(nu)-net.nodesU{ell}];
        dtNcsMdl.E{ell} =  [zeros(np,ny)                                  zeros(np,nu);
                          Bc{ell}*net.nodesY{ell}                       zeros(nc,nu);
                          net.nodesY{ell}                               zeros(ny,nu);
                          -(Dc{ell}+Cc{ell}*Bc{ell})*net.nodesY{ell}    net.nodesU{ell}];  
                      
        dtNcsMdl.B{ell} = [eye(np);zeros(nc,np);-Cp;zeros(nu,np)];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np);zeros(nc,np);-Cp;zeros(nu,np)]];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np);zeros(nc,np);-Cp;zeros(nu,np)]];

        dtNcsMdl.C{ell} = [eye(np) zeros(np,nc+nu+ny)];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [Bp*Dc{ell}*Cp Bp*Cc{ell} Bp*Dc{ell} Bp]];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np,np+nc+ny) -Bp*net.nodesU{ell}]];
        
        dtNcsMdl.D{ell} = dtNcsMdl.B{ell};
        
        dtNcsMdl.J{ell} = zeros((np),nu+ny);
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; zeros((np),nu+ny)];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; [zeros(np,ny) Bp*net.nodesU{ell}]];

    end
end

function [dtNcsMdl,GammaMatrix] = setup3(ncs,verbose)
%----------------------------------------------------------------------
% continuous-time controller and only y is communicated
    
    if verbose==1
        disp('  Controller:           Continuous-time Dynamic Feedback')    
        disp('  Channels Transmitted: Only outputs')
    end
    
    net=ncs.getNetwork;
    p=ncs.getPlant;
    c=ncs.getController;
    
    GammaMatrix = net.nodesY;
    nrNodes = length(net.nodesY);

    % System Matrices
    Ap = p.A;
    Bp = p.B;
    Cp = p.C;
    
    Ac=cell(1,nrNodes);
    Bc=cell(1,nrNodes);
    Cc=cell(1,nrNodes);
    Dc=cell(1,nrNodes);

    % Matrix sizes
    np = size(Ap,1);
    ny = size(Cp,1);
    
    if length(c.A) == 1
       for ctr=1:nrNodes
           Ac{ctr} = c.A{1};
           Bc{ctr} = c.B{1};
           Cc{ctr} = c.C{1};
           Dc{ctr} = c.D{1};
       end
        dtNcsMdl.Atilde{1} = [Ap                      Bp*Cc{1}
                           zeros(size(Ac{1},1),np) Ac{1}];   
    else
        Ac=c.A;
        Bc=c.B;
        Cc=c.C;
        Dc=c.D;
        for ctr=1:nrNodes
            dtNcsMdl.Atilde{1} = [Ap                      Bp*Cc{ctr}
                            zeros(size(Ac{ctr},1),np) Ac{ctr}];
        end
    end
    
    for ell = 1:nrNodes
     
        nc = size(Ac{ell},1);

        % Matrices closed loop system
        B = [Bp*Dc{ell};Bc{ell}];
        C = [Cp zeros(ny,nc)];
        D = eye(ny);

        dtNcsMdl.A{ell} = [zeros(np+nc,np+nc) zeros(np+nc,ny); C eye(ny,ny)-D\GammaMatrix{ell}];
        dtNcsMdl.E{ell} = [zeros(np+nc,ny);D\GammaMatrix{ell}];
        
        dtNcsMdl.B{ell} = [eye(np+nc);-C];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np+nc);-C]];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np+nc);-C]];

        dtNcsMdl.C{ell} = [eye(np+nc) zeros(np+nc,ny)];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [B*D*C B*D]];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np+nc) -B*GammaMatrix{ell}]];
        
        dtNcsMdl.D{ell} = dtNcsMdl.B{ell};
        
        dtNcsMdl.J{ell} = zeros((np+nc),ny);
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; zeros((np+nc),ny)];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; B*GammaMatrix{ell}];

    end

end

function [dtNcsMdl,GammaMatrix] = setup4(ncs,verbose)
    %----------------------------------------------------------------------
    % state-feedback controller (delays can be lumped)
    if verbose==1
        disp('  Controller:           Static Feedback ')
        disp('  Channels Transmitted: Only outputs')
    end
    
    net=ncs.getNetwork;
    p=ncs.getPlant;
    c=ncs.getController;
    nrNodes = length(net.nodesY);
    
    GammaMatrix = cell(0);
    for jj = 1:nrNodes
        GammaMatrix{jj} = blkdiag(net.nodesY{jj},net.nodesU{jj});
    end
    
    nrNodes = length(net.nodesY);

    % System Matrices
    Ap = p.A;
    Bp = p.B;
    Cp = p.C;
    
    Kx=cell(1,nrNodes);
    if length(c.K) == 1
       for ctr=1:nrNodes
           Kx{ctr} = c.K{1};
       end
    else
        Kx=c.K;
    end
    
    % Matrix sizes
    [np nu] = size(Bp);
    ny = size(Cp,1);
    
    % UNDER CONSTRUCTION
    % Construct matrix Lambda
    dtNcsMdl.Atilde{1} = Ap;

    Iy=eye(size(net.nodesY{1}));
    Iu=eye(size(net.nodesU{1}));
    
    %transformation matrix to turn into network-induced error form
    T=cell(nrNodes);
    iT=cell(nrNodes);
    for ell = 1:nrNodes
        gamY=net.nodesY{ell};
        T{ell}=[eye(np)           zeros(np,nu)        zeros(np,ny);
                -Cp               eye(ny)             zeros(ny,nu);
                Kx{ell}*gamY*Cp   Kx{ell}*(Iy-gamY)  eye(nu)];
        iT{ell}=inv(T{ell});
    end    
    
    
    for ell = 1:nrNodes
        gamY=net.nodesY{ell};
        gamU=net.nodesU{ell};
        
        % model with states  \hat{u}_{k-1} and \hat{y}_{k-1}
        dtNcsMdl.A{ell} = [zeros(np)                zeros(np,nu)                zeros(np,ny); 
                           gamY*Cp                  (Iy-gamY)                   zeros(ny,nu);
                           -gamU*Kx{ell}*gamY*Cp     -gamU*Kx{ell}*(Iy-gamY)      (Iu-gamU)];
        dtNcsMdl.E{ell} = zeros(np+nu+ny);
        
        dtNcsMdl.B{ell} = [eye(np); zeros(nu+ny,np)];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np); zeros(nu+ny,np)]];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np); zeros(nu+ny,np)]];
        
        dtNcsMdl.C{ell} = [eye(np)  zeros(np,nu+ny)];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np,np+ny) Bp]];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [-Bp*gamU*Kx{ell}*gamY*Cp  -Bp*gamU*Kx{ell}*(Iy-gamY) -Bp*gamU]];
        
        dtNcsMdl.D{ell} = dtNcsMdl.B{ell};
        
        dtNcsMdl.J{ell} = [zeros(np) zeros(np,nu+ny)];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; [zeros(np) zeros(np,nu+ny)]];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; [zeros(np) zeros(np,nu+ny)]];               
        
        %apply linear transformation to get into e^u_k, e^y_k form
        if ell==nrNodes
            ellPlus=1;
        else
            ellPlus=ell+1;
        end
        dtNcsMdl.A{ell} = T{ellPlus}*dtNcsMdl.A{ell}*iT{ell};
        dtNcsMdl.B{ell} = T{ellPlus}*dtNcsMdl.B{ell};
        dtNcsMdl.C{ell} = dtNcsMdl.C{ell}*iT{ell};
        dtNcsMdl.E{ell} = T{ellPlus}*dtNcsMdl.E{ell};
        dtNcsMdl.D{ell} = T{ellPlus}*dtNcsMdl.D{ell};        
    end

%             for ell = 1:nrNodes
%                 Gamma = blkdiag(ncs.nodesY{ell},ncs.nodesU{ell});
%                 dtNcsMdl.Eb1 = [eye(np);-C];
%                 dtNcsMdl.Eb2 = [eye(np);-C];
%                 dtNcsMdl.Eb3 = [eye(np);-C];
% 
%                 dtNcsMdl.Ec1{ell} = [eye(np) zeros(np,mp+lp)];
%                 dtNcsMdl.Ec2{ell} = [B*Kx*C B*Kx];
%                 dtNcsMdl.Ec3{ell} = [zeros(np) -B*Kx*Gamma];
% 
%                 % add terms in ncs model which are not overapproximated
%                 dtNcsMdl.Ea0{ell} = [zeros(np,np) zeros(np+nc,mp+lp); C eye(mp+lp,mp+lp)-Kx\Gamma];
%                 dtNcsMdl.Eb0{ell} = [zeros(np);Kx\GammaMatrix{ell}];
%             end

end
    
function [dtNcsMdl,GammaMatrix] = setup5(ncs,verbose)
    %----------------------------------------------------------------------
    % DT observer-based controller (delays can be lumped?)
    % Not working if Ap != Ac
    
    if verbose==1
        disp('  Controller:           DT Observer-Based Feedback')
        disp('  Channels Transmitted: All inputs and outputs')
    end
    
    net=ncs.getNetwork;
    p=ncs.getPlant;
    c=ncs.getController;
    
    GammaMatrix = cell(0);
    
    nrNodes = length(net.nodesU);
    for jj = 1:nrNodes
        GammaMatrix{jj} = blkdiag(net.nodesY{jj},net.nodesU{jj});
    end
    
    % System Matrices
    
    Ap = p.A;
    Bp = p.B;
    Cp = p.C;

    
    % Matrix sizes
    [np nu] = size(Bp);
    ny      = size(Cp,1); 
    
    
    Ac=cell(1,nrNodes);
    Bc1=cell(1,nrNodes);
    Bc2=cell(1,nrNodes);
    Cc=cell(1,nrNodes);
    Kx=cell(1,nrNodes);
    Ku=cell(1,nrNodes);
    L=cell(1,nrNodes);

    if length(c.A) == 1
       for ctr=1:nrNodes
           Ac{ctr}  = c.A{1};
           Bc1{ctr} = c.B{1}(:,1:nu);
           Bc2{ctr} = c.B{1}(:,nu+1:end);
           Cc{ctr}  = c.C{1};
           Kx{ctr}  = c.K{1}(:,1:np);
           Ku{ctr}  = c.K{1}(:,np+1:end);
           L{ctr}   = c.L{1};
       end
    else
        Ac  = c.A;        
        Cc  = c.C;
        L   = c.L;
        for ctr=1:nrNodes
            Bc1{ctr} = c.B{ctr}(:,1:nu);
            Bc2{ctr} = c.B{ctr}(:,nu+1:end);
            Kx{ctr}  = c.K{ctr}(:,1:np);
            Ku{ctr}  = c.K{ctr}(:,np+1:end);
        end

    end
    nc      = size(Ac{1},1);
    
    % Construct matrix Lambda
    dtNcsMdl.Atilde{1} = Ap;

    Iy = eye(size(net.nodesY{1},1));
    Iu = eye(size(net.nodesU{1},1));
    
    %transformation matrix to turn into network-induced error form
    T=cell(nrNodes);
    iT=cell(nrNodes);
    for ell = 1:nrNodes
        T{ell}=[eye(np)          zeros(np,nc)       zeros(np,nu)        zeros(np,ny);
                zeros(nc,np)     eye(nc)            zeros(nc,nu)        zeros(nc,ny);
                -Cp              zeros(ny,nc)       zeros(ny,nu)        eye(ny);
                zeros(nu,np)     -Kx{ell}           -Ku{ell}+eye(nu)    zeros(nu,ny)];
        iT{ell}=inv(T{ell});
    end  
    
    for ell = 1:nrNodes
            
        gamY = net.nodesY{ell};
        gamU = net.nodesU{ell};
        
        dtNcsMdl.A{ell} = [zeros(np)       zeros(np,nc)                                    zeros(np,nu)                                       zeros(np,ny);
                          L{ell}*gamY*Cp   Ac{ell}+Bc1{ell}*gamU*Kx{ell}-L{ell}*Cc{ell}     Bc1{ell}*(Iu-gamU)+Bc1{ell}*gamU*Ku{ell}+Bc2{ell}  L{ell}*(Iy-gamY);
                          zeros(nu,np)     gamU*Kx{ell}                                     gamU*Ku{ell}+(Iu-gamU)                             zeros(nu,ny);
                          gamY*Cp          zeros(ny,nc)                                     zeros(ny,nu)                                       (Iy-gamY)];
        dtNcsMdl.E{ell} = blkdiag(zeros(np),zeros(nc),zeros(nu),zeros(ny));
        
        dtNcsMdl.B{ell} = [eye(np); zeros(nc,np); zeros(nu,np); zeros(ny,np)];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np); zeros(nc,np); zeros(nu,np); zeros(ny,np)]];
        dtNcsMdl.B{ell} = [dtNcsMdl.B{ell} [eye(np); zeros(nc,np); zeros(nu,np); zeros(ny,np)]];

        dtNcsMdl.C{ell} = [eye(np)   zeros(np,nc)                        zeros(np,nu)        zeros(np,ny)];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np) zeros(np,nc)      Bp                           zeros(np,ny)]];
        dtNcsMdl.C{ell} = [dtNcsMdl.C{ell}; [zeros(np) Bp*gamU*Kx{ell}  -Bp*gamU*(Iu-Ku{ell})      zeros(np,ny)]];

        dtNcsMdl.D{ell} = dtNcsMdl.B{ell};
        
        dtNcsMdl.J{ell} = [zeros(np) zeros(np,nc)    zeros(np,nu)         zeros(np,ny)];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; [zeros(np) zeros(np,nc)    zeros(np,nu)  zeros(np,ny)]];
        dtNcsMdl.J{ell} = [dtNcsMdl.J{ell}; [zeros(np) zeros(np,nc)    zeros(np,nu)  zeros(np,ny)]];              
        
        %apply linear transformation to get into e^u_k, e^y_k form
        if ell==nrNodes
            ellPlus=1;
        else
            ellPlus=ell+1;
        end
        dtNcsMdl.A{ell} = T{ellPlus}*dtNcsMdl.A{ell}*iT{ell};
        dtNcsMdl.B{ell} = T{ellPlus}*dtNcsMdl.B{ell};
        dtNcsMdl.C{ell} = dtNcsMdl.C{ell}*iT{ell};
        dtNcsMdl.E{ell} = T{ellPlus}*dtNcsMdl.E{ell};
        dtNcsMdl.D{ell} = T{ellPlus}*dtNcsMdl.D{ell}; 
    end

end


