function polyOvrAprx = genPolyOvrAprx(dtNcsMdl,ovrAprxType,varargin)
%GENPOLYOVRAPRX  From a discrete-time NCS model (type 'help genDtNcsModel.m'), 
%                this functions generates a polytopic overapproximation of 
%                the form 
%
%    x_{k+1} = (\sum_{m=1}^{M}alpha^m_k* Ab{l,m} + Bb{l}* Delta_k *Cb{l})x_{k} +
%               (\sum_{m=1}^{M}alpha^m_k* Eb{l,m} + Db{l}* Delta_k *Jb{l})w_{k}
%
%               where x_k is the value of the NCS state vector at time k, w_k 
%               is a disturbance and Delta_k belongs to a set of structured 
%               norm bounded matrices. See [Bauer et. al. NECSYS, 2012] eqn 
%               (2) for more info. Overapproximation procedures are based on:
% 
%                   GNB - [Donkers et. al., Trans. Autom. Control 2011]
%                   JNF - [Cloosterman et. al., Trans. Autom. Control 2009]
%                   CH  - [Gielen et. al., Automatica 2010]
%
%   POLYOVRAPRX = GENPOLYOVRAPRX(DTNCSMDL,OVRAPRXTYPE,EPSU,GRIDPTM,PLOTFLAG,CUSTOMGRIDPTS) 
%   Generates the polytopic overapproximation POLYOVRAPRX from the data in 
%   DTMDL using the overapprimation OVRAPRXTYPE.
%
%   INPUTS
%   ------------------------------------------------------
%      DTNCSMDL         - strucutre containing discrete-time model info 
%                         from genDtNcsModel.m.  All fields specified in the 
%                         output of genDtNcsModel.m are required fields of DTNCSMDL.
%      OVRARPXTYPE      - 'GNB', 'JNF' or 'CH'
%      EPSU             - (GNB only) Desired overapproximation tightness parameter 
%      GRIDPTMAX        - (GNB only) Maximum number of gridpoints
%      PLOTFLAG         - (GNB only) 'true' plots gridpoints as algorithm runs
%                          else 'false' suppresses plotting
%      VERBOSE          - 0 to suppress text output
%                         
%   OUTPUTS
%   ------------------------------------------------------
%      POLYOVRAPRX   - strucuture containing information about
%                      the overapproximated model
%                       ~~~~Fields~~~~~~~~~~~~
%                       POLYOVRAPRX.type        - 'GNB', 'JNF' or 'CH'                       
%                       POLYOVRAPRX.Ab          - Ab{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Bb          - Bb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Cb          - Cb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Db          - Db{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Eb          - Eb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.Jb          - Jb{l} matrices in overapprox. eqn
%                       POLYOVRAPRX.JordanBlks  - Jordan blocks used for overapproximation
%                       POLYOVRAPRX.alphas      - (JNF,CH) list of symbolic alpha terms 
%                       POLYOVRAPRX.alphaMinMax - (JNF,CH) min and max of alphas used 
%                                                 to create the overapproximation
%                       POLYOVRAPRX.gridpts     - (GNB) list of grid points in (h,tau) space                       
%                       POLYOVRAPRX.trigangles  - (GNB) list of triangles in (h,tau) space
%                       POLYOVRAPRX.deltas      - (GNB) overapproximation deltas per 
%                                                 triangle per jordan block 
%                       POLYOVRAPRX.MaxEps      - (GNB) maximum value of epsilon 
%                       POLYOVRAPRX.MinEps      - (GNB) minimum value of epsilon 
%
%   Customization Tips:
%      *The structure which is input, DTNCSMODEL, can be generated by a 
%       custom discrete-time NCS model of the form specified in eqn (1) of
%       [Bauer et. al. NECSYS, 2012].
%      *The structure which is output, POLYOVRAPRX, is suitable for use in 
%       custom performance or stability analysis conditions.
%
%   See also GENDTNCSMODEL
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if ~isfield(dtNcsMdl,'A')
        error('genPolyOvrAprx.m: A is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'B')
        error('genPolyOvrAprx.m: B is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'C')
        error('genPolyOvrAprx.m: C is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'D')
        error('genPolyOvrAprx.m: D is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'E')
        error('genPolyOvrAprx.m: E is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'J')
        error('genPolyOvrAprx.m: J is required to be a field of the input variable DTNCSMDL.')
    elseif ~isfield(dtNcsMdl,'tPoly')
        error('genPolyOvrAprx.m: tPoly is required to be a field of the input variable DTNCSMDL.')
    end    

    if strcmp(ovrAprxType,'JNF') || strcmp(ovrAprxType,'CH')
        if nargin == 2
            polyOvrAprx = genPolyOvrAprxJNFCH(dtNcsMdl,ovrAprxType,1);
        elseif nargin == 3
            polyOvrAprx = genPolyOvrAprxJNFCH(dtNcsMdl,ovrAprxType,varargin{1});
        else
            error('genPolyOvrAprx.m: Only two inputs are required for JNF or CH: DTMDL and OVRAPRXTYPE.')
        end
    elseif strcmp(ovrAprxType,'GNB')        
        if nargin == 5
            polyOvrAprx = genPolyOvrAprxGNB(dtNcsMdl,varargin{1},varargin{2},varargin{3},1);    
        elseif nargin == 6
            polyOvrAprx = genPolyOvrAprxGNB(dtNcsMdl,varargin{1},varargin{2},varargin{3},varargin{4});    
        else
            error('genPolyOvrAprx.m: Incorrect number of input arguements for GNB.')
        end
    else
        error('genPolyOvrAprx.m: Cannot create polytopic overapproximation.  Overapproimation type is unknown.')
    end
    

end

function polyOvrAprx = genPolyOvrAprxJNFCH(dtNcsMdl,ovrAprxType,verbose)
% Generates polytopic overapproximation of NCS object using JNF or CH.
% Written by:  Nick Bauer
    
    if verbose==1
        disp(' ')
        disp(['OVERAPPROXIMATION DATA: ' ovrAprxType])
        disp('*OVERAPPROXIMATION USING MIN, MAX TIMING BOUNDS*') 
    end
    
    hmin    = min(dtNcsMdl.tPoly(:,1));
    hmax    = max(dtNcsMdl.tPoly(:,1));
    taumin  = min(dtNcsMdl.tPoly(:,2));
    taumax  = max(dtNcsMdl.tPoly(:,2));
    
    nrNodes = length(dtNcsMdl.A);
    syms hk hktk                    % symbols used in the exponential and integral terms
    
    %% Compute Exponential and Integral Terms in ncs Model
   
    S11xl = cell(nrNodes,1);
    S22xl = cell(nrNodes,1);
    S33xl = cell(nrNodes,1);
    S44xl = cell(nrNodes,1);
    S55xl = cell(nrNodes,1);
    S66xl = cell(nrNodes,1);
    
    JJ=cell(0);
    totalSs = 0;
    
    for ell=1:nrNodes
        
        if ell <= length(dtNcsMdl.Atilde)  % checks if dtNcsMdl.Atilde is node dependent, if so compute new overapproximation

            S11     = cell(0);
            S22     = cell(0);
            S33     = cell(0);
            alfa11  = cell(0);
            alfa22  = cell(0);
            alfa33  = cell(0);

            [J Q] = rjd(dtNcsMdl.Atilde{ell});   % determine the real Jordan form with the Linear Systems Toolbox
            JJ{ell} = sepJdnBlocks(J);    % Determination of separate Jordan blocks with real eigenvalues

            q = 1; r = 1;
            for i = 1:length(JJ{ell})
                % compute e^(JJ{ell}{i}*hk) = sum(S1{i}*alfa1{i})
                [S1 alfa1] = jnf(JJ{ell}{i});                    % Determination of varying factors from exponent

                % add the part e^(JJ{ell}{i}*hk) = sum(S1{i}*alfa1{i}) into e^(A*hk) = sum(S11{i}*alfa11{i})
                q = q + size(JJ{ell}{i},1);
                for j = 1:size(S1,2)
                    S11{r} = Q*blkdiag(zeros(q-1-size(JJ{ell}{i},1)),S1{j},zeros(size(J,1)-q+1))/Q;
                    alfa11{ell,r} = alfa1{j};
                    r = r+1;
                end
            end

            % Transform S11,alfa11 to Cayley-Hamilton terms
            if strcmp(ovrAprxType,'CH') || strcmp(ovrAprxType,'CHe')
                [S11 alfa11(ell,:)] = jnf2ch(dtNcsMdl.Atilde{ell},S11,alfa11(ell,:));                   
            end

            % compute int_0^hk(e^{A*s}ds)= sum(S22{i}*alfa22{i}) using S11 and alfa11 computed above
            if rank(dtNcsMdl.Atilde{ell}) ~= size(dtNcsMdl.Atilde{ell},2)                         % A not invertible
                 for ctr = 1:size(S11,2)
                    alfa22{ell,ctr} = int(alfa11{ell,ctr},hk);                     %need to use int() since Cayley can have terms like f1 + f2*hk^2
                    S22{ctr} = S11{ctr};
                    alfa22{ell,ctr+size(S11,2)} = subs(alfa22{ell,ctr},hk,0,0);    %need to use int() since Cayley can have terms like f1 + f2*hk^2
                    S22{ctr+size(S11,2)} = -S11{ctr};
                end   
            else                                        % A invertible,use int_hktk^hk(e^{A*s}ds) = inv(A)*int(e^A*hk) - inv(A)*int(e^A*hktk)
                for ctr = 1:size(S11,2)
                    alfa22{ell,ctr} = alfa11{ell,ctr};
                    S22{ctr} = dtNcsMdl.Atilde{ell}\S11{ctr};
                    alfa22{ell,ctr+size(S11,2)} = subs(alfa11{ell,ctr},hk,0,0);
                    S22{ctr+size(S11,2)} = -dtNcsMdl.Atilde{ell}\S11{ctr};
                end
            end

            % compute int_0^hktk(e^{A*s}ds)= sum(S33{i}*beta33{i}) using S33 and
            % alfa33 computed above
            for j = 1:(size(S22,2))
                alfa33{ell,j} =subs(alfa22{ell,j},hk,hktk);             % phi(k+1) = [expm(A*h) int(expm(A*s),s,0,h); zeros(n) 1]*phi(K)
                S33{j} = S22{j};
            end
        else   %Atilde is not node dependent
            JJ(ell) = JJ(1);
            alfa11(ell,:) = alfa11(1,:);
            alfa22(ell,:) = alfa22(1,:);
            alfa33(ell,:) = alfa33(1,:);
            % S11, S22, S33 stay the same as well
        end
        
        %-------------------------------------------------------------------
        % Place the S11,S22,S33 matrices in closed-loop form.
        nz = size(dtNcsMdl.Atilde{1},1);
        for i = 1:size(S11,2)
            S11xl{ell,i} = dtNcsMdl.B{ell}(:,1:nz)*S11{i}*dtNcsMdl.C{ell}(1:nz,:);
            S44xl{ell,i} = dtNcsMdl.D{ell}(:,1:nz)*S11{i}*dtNcsMdl.J{ell}(1:nz,:);
        end

        for i = 1:size(S22,2)
            S22xl{ell,i} = dtNcsMdl.B{ell}(:,nz+1:2*nz)*S22{i}*dtNcsMdl.C{ell}(nz+1:2*nz,:);
            S55xl{ell,i} = dtNcsMdl.D{ell}(:,nz+1:2*nz)*S22{i}*dtNcsMdl.J{ell}(nz+1:2*nz,:);
        end

        for i = 1:size(S33,2)
            S33xl{ell,i} = dtNcsMdl.B{ell}(:,2*nz+1:3*nz)*S33{i}*dtNcsMdl.C{ell}(2*nz+1:3*nz,:);
            S66xl{ell,i} = dtNcsMdl.D{ell}(:,2*nz+1:3*nz)*S33{i}*dtNcsMdl.J{ell}(2*nz+1:3*nz,:);
        end
        totalSs = totalSs+size(S11,2)+size(S22,2)+size(S33,2);
        
    end
    
    %% Combine alfa_i*S_i Exponential and Integral terms to create overapproximation F matrix

    alfaStack = cell(0);     % stack all the alfa terms
    Ab_stack = cell(nrNodes,totalSs);
    Eb_stack = num2cell(zeros(nrNodes,totalSs));
    numSs = cell(1,nrNodes);
    for ell=1:nrNodes
        
        nCl = size(dtNcsMdl.A{1},1);
        nw = size(dtNcsMdl.E{1},2);
        numSs{ell} = size(S11xl,2)+size(S22xl,2)+size(S33xl,2);
        
        % stack alfas by ell
        alfaStack = [alfaStack alfa11(ell,:) alfa22(ell,:) alfa33(ell,:)];  %#ok<AGROW>
        if ell==1
            zeroPadBeforeAb = [];
            zeroPadBeforeEb = [];
        else
            topMarker = sum([numSs{1:ell-1}]);  %mark how many zeros are needed at top
            zeroPadBeforeAb = mat2cell(zeros(nCl,nCl*(topMarker)),nCl,nCl*ones(topMarker,1));
            zeroPadBeforeEb = mat2cell(zeros(nCl,nw*(topMarker)),nCl,nw*ones(topMarker,1));
        end
        
        if ell==nrNodes || nrNodes == 1
            zeroPadAfterAb = [];
            zeroPadAfterEb = [];
        else
            marker = sum([numSs{1:ell}]);       %mark how many zeros are needed at bottom
            zeroPadAfterAb  = mat2cell(zeros(nCl,nCl*(totalSs-marker)),nCl,nCl*ones(totalSs-marker,1));
            zeroPadAfterEb  = mat2cell(zeros(nCl,nw*(totalSs-marker)),nCl,nw*ones(totalSs-marker,1));        
        end
        Ab_stack(ell,:) =   [zeroPadBeforeAb S11xl(ell,:) S22xl(ell,:) S33xl(ell,:) zeroPadAfterAb];
        Eb_stack(ell,:) =   [zeroPadBeforeEb S44xl(ell,:) S55xl(ell,:) S66xl(ell,:) zeroPadAfterEb];
    end
    
    % remove constant and duplicate alfa terms from list of alfas
    [alfa_ht,Ab_ht,Aconst,Eb_ht,Econst] = reduceAlphas(alfaStack,Ab_stack,Eb_stack,[hmin hmax],[taumin taumax]);

    % find min and max of alfa terms
    [alfaComboSet,alfaMinMax] = getAlphaMinMax(alfa_ht,[hmin hmax],[taumin taumax]);
    
    %% Build Closed-Loop Overapproximation

    %disp('eigenvalues of A = ')
    %disp(num2str(eig(A)'))
    if verbose == 1
        disp(['varying alfa terms = ' num2str(size(alfa_ht,2))])
        for ctr = 1:length(alfa_ht)
            disp(['   alpha' num2str(ctr) ' = ' char(alfa_ht{ctr})]);
        end
        disp(' ')
    end

    %----------------------------------------------------------------
    % Construct open-loop vertices of overapproximation
    switch ovrAprxType
        case {'JNF', 'CH'}
            % Using alfaComboSet gives 2^(2*Nu) vertices of the polytope
            Ab = cell(1,size(alfaComboSet,2));
            Eb = cell(1,size(alfaComboSet,2));
            if size(alfaComboSet,2)==0
                for ell=1:nrNodes
                    Ab{ell,1} = Aconst{ell}+dtNcsMdl.A{ell};
                    Eb{ell,1} = Econst{ell}+dtNcsMdl.E{ell};
                end
            else
                for ell=1:nrNodes
                    for i = 1:size(alfaComboSet,2)
                            Ab{ell,i} = Aconst{ell}+dtNcsMdl.A{ell};
                            Eb{ell,i} = Econst{ell}+dtNcsMdl.E{ell};
                        for j = 1:size(alfaComboSet{i},2)                       %multiply matrices Fi and Gi by all values alfaComboSet{i} to make a polytopic vertex
                            Ab{ell,i} = Ab{ell,i} + alfaComboSet{i}(j)*Ab_ht{ell,j};
                            Eb{ell,i} = Eb{ell,i} + alfaComboSet{i}(j)*Eb_ht{ell,j};
                        end
                    end
                end
            end

        case 'CHe'
            % Using the alfaMinMax output results in reduced 2*2*Nu vertices of the polytope
            Ab = cell(1,size(alfaMinMax,1)*size(alfaMinMax,2));
            Eb = cell(1,size(alfaMinMax,1)*size(alfaMinMax,2));

            if size(alfaMinMax,2) ==0   %Nu = 0
                for ell=1:nrNodes
                    Ab{ell,1} = Aconst{ell}+dtNcsMdl.A{ell};
                    Eb{ell,1} = Aconst{ell}+dtNcsMdl.E{ell};
                    
                end
            else
                p=1;
                for ell=1:nrNodes
                    for i = 1:size(alfaMinMax,1)
                        for j = 1:size(alfaMinMax,2)
                            Ab{ell,p} = Aconst{ell}+dtNcsMdl.A{ell}+(Nu+1)*(alfaMinMax(i,j)*Ab_ht{ell,j});
                            Eb{ell,p} = Econst{ell}+dtNcsMdl.E{ell}+(Nu+1)*(alfaMinMax(i,j)*Eb_ht{ell,j});
                            p = p+1;
                        end
                    end
                end
            end
    end
    
    %count the number of polytopic vertices
    totalVerts = size(Ab,2);
    
    if verbose==1
        disp(['      Number of Polytopic Vertices = ' num2str(totalVerts)])
    end
    
    polyOvrAprx.type = ovrAprxType;
    polyOvrAprx.alphas = alfa_ht;
    polyOvrAprx.alphaMinMax = alfaMinMax;
    polyOvrAprx.JordanBlks = JJ;                 % Jordan Blocks used for overapproximation
    polyOvrAprx.indices = '(nodes,vertices)'; 
    
    polyOvrAprx.Ab = Ab;
    polyOvrAprx.Eb = Eb;
    
    for ell=1:nrNodes
        if length(dtNcsMdl.Atilde) == 1
            Atilde = dtNcsMdl.Atilde{1};
        else
            Atilde=dtNcsMdl.Atilde{ell};
        end
        
       % JNF and CH do not have any uncertainty, so set Bb,Cb,Db,Jb to zero
       polyOvrAprx.Bb{ell} = zeros(size(Ab{ell},1),3*size(Atilde,1));
       polyOvrAprx.Db{ell} = zeros(size(Ab{ell},1),3*size(Atilde,1));
       polyOvrAprx.Cb{ell} = zeros(3*size(Atilde,1),size(Ab{ell},1));
       polyOvrAprx.Jb{ell} = zeros(3*size(Atilde,1),size(Ab{ell},1));

    end
       
end


function polyOvrAprx = genPolyOvrAprxGNB(dtNcsMdl,epsU,gridPtMax,plotflag,verbose)
% Generates polytopic overapproximation of NCS object using GNB.
% Written by:  Bas van Loon
            
    if ~isscalar(epsU) || epsU < 0
        error('genPolyOvrAprx.m: EPSU must be a scalar number greater than zero.')
    elseif ~isscalar(gridPtMax) || gridPtMax < 0
        error('genPolyOvrAprx.m: GRIDPNTM must be an integer greater than zero.')
    end

    if verbose==1
        disp(' ')
        disp('OVERAPPROXIMATION DATA: GNB')
        disp('*OVERAPPROXIMATION USING TIMING UNCERTAINTY POLYGON*') 
    end
    nrNodes = length(dtNcsMdl.A);
    customGridPts = dtNcsMdl.tPoly;
%% - Construct Bb, Cb, and Fb
    
    % Matrix Cb and Jb of overapproximation
    Bb = cell(1,nrNodes);
    Cb = cell(1,nrNodes);
    Db = cell(1,nrNodes);
    Jb = cell(1,nrNodes);
    JJ = cell(0);
    J = cell(0);
    T=cell(0);
    
    if length(dtNcsMdl.Atilde)==1     %Atilde not node dependent
        [J{1} T{1}]= rjd(dtNcsMdl.Atilde{1});
        JJ{1} = sepJdnBlocks(J{1});             % NCS toolbox function
            
        for ell = 1:nrNodes
            Ttemp = blkdiag(T{1},T{1},T{1});
            Bb{ell} = dtNcsMdl.B{ell}*Ttemp;
            Db{ell} = dtNcsMdl.D{ell}*Ttemp;
            Cb{ell} = Ttemp\dtNcsMdl.C{ell};
            Jb{ell} = Ttemp\dtNcsMdl.J{ell};
        end
    else                        %Atilde is node dependent
        for ell = 1:nrNodes    
            [J{ell} T{ell}]= rjd(dtNcsMdl.Atilde{ell});
            JJ{ell} = sepJdnBlocks(J{ell});             % NCS toolbox function
            Ttemp = blkdiag(T{ell},T{ell},T{ell});
            Bb{ell} = dtNcsMdl.B{ell}*Ttemp;
            Db{ell} = dtNcsMdl.D{ell}*Ttemp;
            Cb{ell} = Ttemp\dtNcsMdl.C{ell};
            Jb{ell} = Ttemp\dtNcsMdl.J{ell};
        end
    end     

%% Gridding procedure

    %determine if const, line or area
    wiggleRoom = 1e-12;
    if ~isempty(customGridPts)
        
        if any((customGridPts(:,1)-customGridPts(:,2))<0)
            error('tPoly points do not satisfy small delay assumption')
        end
        
        if all(abs(diff(customGridPts(:,1)))<wiggleRoom) && all(abs(diff(customGridPts(:,2)))<wiggleRoom)
            setG = 'const';
        elseif polyarea(customGridPts(:,1),customGridPts(:,2))==0
            setG = 'line';
        else
            setG='area';
        end 
    end
    
    %2- Compute overapproximation
    % Combine input data
    GNBinput.Bb        = Bb;
    GNBinput.Cb        = Cb;
    GNBinput.JJ        = JJ;
    GNBinput.gridpnt0  = customGridPts;
    GNBinput.gridpntM  = gridPtMax;
    GNBinput.epsilon_u = epsU;
    GNBinput.nrNodes   = nrNodes;
    GNBinput.setG      = setG;
    GNBinput.plotflag  = plotflag;
    GNBinput.verbose   = verbose;
    % Gridding function
    [GNBoutput] = gnb(GNBinput);

    % Extract output data, matrix Bbd of overapproximation
    Bb      = GNBoutput.Bbd;
    gridpnt  = GNBoutput.gridpnt;
    
    % Total number of gridpoints
    NrGridpnt = size(gridpnt,1);

%% -- Determine Ah, Eh and Eht evaluated at the gridpoints

    Ah = cell(nrNodes,NrGridpnt);
    Eh = cell(nrNodes,NrGridpnt);
    Eht = cell(nrNodes,NrGridpnt);
    Ab = cell(nrNodes,NrGridpnt); 
    Eb = cell(nrNodes,NrGridpnt);
    
    for ell=1:nrNodes

        if length(dtNcsMdl.Atilde)==1
            J{ell}=J{1};
            T{ell}=T{1};
            JJ{ell}=JJ{1};
        end
        
        nrJdnBlks   = length(JJ{ell});       % total number of Jordan blocks    
        % Search for the maximum size of the Jordan Blocks
        Msize = zeros(nrJdnBlks,1);
        for q = 1:nrJdnBlks
            Msize(q,1) =size(JJ{ell}{q},1);
        end

        if max(Msize)== 1
            JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
        else
            JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
        end

        switch JordanB
            case {'RD'} % diagonalizable, avoid computing integral of matrix expm
                Ehint = zeros(nrJdnBlks,nrJdnBlks);
                Ehtint = zeros(nrJdnBlks,nrJdnBlks);
                for m = 1:NrGridpnt
                    Ah{ell,m} = T{ell}*expm(J{ell}*gridpnt(m,1))/T{ell};
                    for q = 1:nrJdnBlks
                        if abs(J{ell}(q,q)) > 1e-14
                            Ehint(q,q)  = 1/J{ell}(q,q)*(exp(J{ell}(q,q)*gridpnt(m,1))-1);
                            Ehtint(q,q) = 1/J{ell}(q,q)*(exp(J{ell}(q,q)*(gridpnt(m,1)-gridpnt(m,2)))-1);
                        else
                            Ehint(q,q)  = gridpnt(m,1);
                            Ehtint(q,q) = gridpnt(m,1)-gridpnt(m,2);
                        end
                    end             
                    Eh{ell,m}  = double(T{ell}*Ehint/T{ell});
                    Eht{ell,m} = double(T{ell}*Ehtint/T{ell});
                    clear Ehint Ehtint
                end       
            case {'RnDC'} % not diagonalizable, use symbolic toolbox
                syms s
                for m = 1:NrGridpnt
                    Ah{ell,m}  = T{ell}*expm(J{ell}*gridpnt(m,1))/T{ell};
                    Eh{ell,m}  = T{ell}*double(int(expm(J{ell}*s),s,0,gridpnt(m,1)))/T{ell};
                    Eht{ell,m} = T{ell}*double(int(expm(J{ell}*s),s,0,gridpnt(m,1)-gridpnt(m,2)))/T{ell};
                end    
        end % end switch JordanB
        
        %construct system gridpoints
        for m = 1:NrGridpnt
            Ab{ell,m} = dtNcsMdl.A{ell} + dtNcsMdl.B{ell}*blkdiag(Ah{ell,m},Eh{ell,m},Eht{ell,m})*dtNcsMdl.C{ell};
            Eb{ell,m} = dtNcsMdl.E{ell} + dtNcsMdl.D{ell}*blkdiag(Ah{ell,m},Eh{ell,m},Eht{ell,m})*dtNcsMdl.J{ell};   
        end
        
    end


%% Ouput

    polyOvrAprx.type = 'GNB';
    polyOvrAprx.gridpts =    GNBoutput.gridpnt;  % list with grid points
    polyOvrAprx.triangles  = GNBoutput.TRI;      % list with triangles
    polyOvrAprx.JordanBlks = JJ;                 % Jordan Blocks used for overapproximation
    polyOvrAprx.deltas =     GNBoutput.deltas;   % overapproximation deltas per triangle per jordan block
    polyOvrAprx.MaxEps =     GNBoutput.MaxEps;   % maximum value of epsilon
    polyOvrAprx.MinEps =     GNBoutput.MinEps;   % minimum value of epsilon
    
    polyOvrAprx.Ab = Ab;
    polyOvrAprx.Bb = Bb;
    polyOvrAprx.Cb = Cb;
    polyOvrAprx.Db = Db;
    polyOvrAprx.Eb = Eb;
    polyOvrAprx.Jb = Jb;
end