function [alfaMinMaxCombos,alfaMinMax] = getAlphaMinMax(alfa,h,tau)
%GETALPHAMINMAX  (JNF and CH overapproximation) calculates the min and max 
%                for each symbolic alfa function using the bounds on the sampling
%                interval and transmission delay.
% 
%   [ALFAMINMAXCOMBOS,ALFAMINMAX] = GETALPHAMINMAX(ALFA,H,TAU)
%   calculates the min and max (ALFAMINMAX) and all combinations of the min 
%   and max (ALFAMINMAXCOMBOS) for each symbolic function ALFA using the 
%   bounds H and TAU.
%
%   See also JNF, JNF2CH
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% --------------------------------------------------------------------------
% compute min and max of alfa_i of the varying part and store in alfaMinMax where 
% alfa_i_max = alfaMinMax(2,i) and alfa_i_min = alfaMinMax(1,i)
% this comuptation is done by gridding the function alfa_var(i) and
% choosing the max value 

    syms hk tk hktk

    alfaMinMax = zeros(2,length(alfa));

    gridPoints = 1000;
    hBound = h(1):(h(2)-h(1))/(gridPoints-1):h(2);
    tBound = tau(1):(tau(2)-tau(1))/(gridPoints-1):tau(2);
    htBound = h(1)-tau(2):(h(2)-tau(1) - (h(1)-tau(2)))/(gridPoints-1):h(2)-tau(1);

    v=version;  % check matlab version, fmincon changed depending on version
    if str2double(v(3:4))>=10    %10 = r2010b
        options=optimset('Display','off','Algorithm','sqp','LargeScale','off');    %'sqp' only in r2010a and later
    elseif str2double(v(3:4))>=7  %7 = r2008b
        options=optimset('Display','off','Algorithm','active-set','LargeScale','off');
    end
    % versions earlier than r2008b cannot use fmincon since 'matlabFunction' is not defined

    for k = 1:length(alfa)
        impl = alfa{k};
        % need 'if' structure since if h or tau is constant, then hBound or tBoutnd is not defined
        if diff(alfa{k},hktk) ~= 0
            impl = subs(impl,hktk,htBound,0);                                                       % grid function by substututing htBound into sym hktk 

            startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
            startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
            if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined
                alfaFunct = matlabFunction(alfa{k});                                                % create matlab function from symbolic expression (only for r2008b and newer)
                minPt = fmincon(alfaFunct,startMin,[],[],[],[],htBound(1),htBound(end),[],options);     % search for local min given starting point
                minAlfaFunct = matlabFunction(-alfa{k});                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
                maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],htBound(1),htBound(end),[],options);  % search for local max given starting point

                alfaMinMax(1,k) = subs(alfaFunct,hktk,minPt);                                           % evaluate value given by fmincon
                alfaMinMax(2,k) = subs(alfaFunct,hktk,maxPt);                                           % evaluate value given by fmincon
            else
                alfaMinMax(1,k) = startMin;
                alfaMinMax(2,k) = startMax;
            end

        elseif diff(alfa{k},hk) ~= 0
            impl = subs(impl,hk,hBound,0);                                                          % grid function by substututing hBound into sym hk

            startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
            startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
            if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined 
                alfaFunct = matlabFunction(alfa{k});                                                % create matlab function from symbolic expression (only for r2008b and newer)
                minPt = fmincon(alfaFunct,startMin,[],[],[],[],hBound(1),hBound(end),[],options);       % search for local min given starting point
                minAlfaFunct = matlabFunction(-alfa{k});                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
                maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],hBound(1),hBound(end),[],options);    % search for local max given starting point

                alfaMinMax(1,k) = subs(alfaFunct,hk,minPt);                                             % evaluate value given by fmincon
                alfaMinMax(2,k) = subs(alfaFunct,hk,maxPt);                                             % evaluate value given by fmincon
            else
                alfaMinMax(1,k) = startMin;
                alfaMinMax(2,k) = startMax;
            end        

        elseif diff(alfa{k},tk) ~= 0
            impl = subs(impl,tk,tBound,0);                                                          % grid function by substututing tBound into sym tk

            startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
            startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
            if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined
                alfaFunct = matlabFunction(alfa{k});                                                % create matlab function from symbolic expression (only for r2008b and newer)
                minPt = fmincon(alfaFunct,startMin,[],[],[],[],tBound(1),tBound(end),[],options);       % search for local min given starting point
                minAlfaFunct = matlabFunction(-alfa{k});                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
                maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],tBound(1),tBound(end),[],options);    % search for local max given starting point

                alfaMinMax(1,k) = subs(alfaFunct,tk,minPt);                                             % evaluate value given by fmincon
                alfaMinMax(2,k) = subs(alfaFunct,tk,maxPt);                                             % evaluate value given by fmincon
            else
                alfaMinMax(1,k) = startMin;
                alfaMinMax(2,k) = startMax;
            end         
        end

    end     

%% ------------------------------------------------------------------------
% use the values computed in alfaMinMax to construct alfaMinMaxCombos{i}, which is a cell
% of vectors.  The set of these vectors is all combinations alfa_min and 
% alfa_max 
%
% The size(alfaMinMaxCombos,2) is the number of polytopic vertices, and each vector alfaMinMaxCombos{i}
% corresponds to the mulipliers to create a single vertex

    alfaMinMaxCombos = cell(0);   

    if size(alfaMinMax,2) == 1          %only one alfa_i
        alfaMinMaxCombos{1} = alfaMinMax(1,:);
        alfaMinMaxCombos{2} = alfaMinMax(2,:);
    elseif size(alfaMinMax,2) > 1

        % use upper and lower cells as matrix sets to get all combinations 
        % of the min and max of alfa_i 

        lower = cell(1,size(alfaMinMax,2));                     
        upper = cell(1,size(alfaMinMax,2));                     

        for i = 1:size(alfaMinMax,2)
            lower{i} = nchoosek(1:size(alfaMinMax,2),i);
            for j = 1:size(lower{i},1)
                upper{i}(j,:) = setdiff(1:size(alfaMinMax,2),lower{i}(j,:));
            end
        end

        z = 1;
        for i = 1:size(lower,2)
            for j = 1:size(lower{i},1)
                alfaMinMaxCombos{z}(1,lower{i}(j,:)) = alfaMinMax(1,lower{i}(j,:));
                alfaMinMaxCombos{z}(1,upper{i}(j,:)) = alfaMinMax(2,upper{i}(j,:));
                z = z+1;
            end
        end
        alfaMinMaxCombos{end+1} = alfaMinMax(2,:);
    end
end
