function [GNBoutput] = gnb(GNBinput)
% GNB The Gridding and Norm Bounding procedure generates a set of gridpoints  
%     and corresponding worst-case approximation errors so that a polytopic 
%     overapproximation can be obtained. An initial set of gridpoints are 
%     input into the alogirthm and then gridpoints are iteratively added at
%     the locations of the worst-case approximation error until either a 
%     user specified tightness is achieved or the maximum number of gridpoints 
%     is reached.
%
%     Most of the theory and notation that is used in this code is based 
%     on : "Stability Analysis of Networked Control Systems Using a Switched  
%     Linear System Approach", by Donkers et al., Trans. Autom. Control 2011 
%     
%     The code itself (and the theory behind the code) is explained in greater 
%     detail in the manual of the NCS Toolbox and in the thesis "Stability 
%     Analysis of Networked and Quantized Control Systems: Theory and Matlab 
%     Implementation", MSc Thesis S.J.L.M. van Loon, Eindhoven University of 
%     Technology 2011
%
%   [GNBOUPUT] = GNB(GNBINPUT) takes the GNBINPUT which is a structure
%   containing relevent information for the overapproximation algorithm and
%   returns GNBOUTPUT which is a structure containing information used to 
%   create a polytopic overapproximation with norm-bounded uncertainty.
%
%-------------------------------------------------------------------------
% The fields of GNBINPUT are:
%   Bb         : matrices used for computing epsilon, related to the B matrix 
%                of the discrete-time model
%   Cb         : matrices used for computing epsilon, related to the C matrix 
%                of the discrete-time model
%   JJ         : cell with real Jordan blocks
%   gridpnt0   : initial set with grid points
%   gridpntM   : maximum number of gridpoints
%   epsilon_u  : user defined tightness of the overapproximation
%   NrNodes    : number of nodes
%   setG       : string that contains information about the case under
%                investigation, i.e., 
%                   # constant delay case
%                   # constant transmission interval case
%                   # the case that both delays as transmission intervals 
%                     are time varying.
%   plotflag   : if plotflag == true, the partitioning of Theta into 
%                trianges is shown in a plot. The triangle with the 
%                largest error appears in blue, after which it is divided 
%                into new triangles
% NOTE: this function tries to achieve the epsilon_u given the constraint 
%       on the maximum number of grid points that are allowed by the user,
%       and the maximum number of iterations, set as 2X the maximum number
%       of grid points (to prevent infinite interations)
%
% -------------------------------------------------------------------------
% The fields of GNBOUTPUT are:
%   Bbd     : matrix B
%   gridpnt : two column vector specifying grid points
%   MaxEps  : maximum value of the resulting epsilon over all nodes
%   MinEps  : minimum value of the resulting epsilon over all nodes
%   TRI     : list of triangles
%   deltas  : list of overapproximation deltas per triangle per 
%             jordan block
%
%`  See also INITGRIDPTS
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% Extract input data
Bb        = GNBinput.Bb;
Cb        = GNBinput.Cb;
JJ        = GNBinput.JJ;
gridpnt0  = GNBinput.gridpnt0;
gridpntM  = GNBinput.gridpntM;
epsilon_u = GNBinput.epsilon_u;
nrNodes   = GNBinput.nrNodes;
setG      = GNBinput.setG;
plotflag  = GNBinput.plotflag;
verbose   = GNBinput.verbose;

%% Determine case
switch lower(setG)
    % constant h and tau (no uncertainty)
    case {'const'}
        if verbose == 1
            disp('  The maximum value of epsilon = 0')
            disp('  Total amount of gridpoints   = 1')
        end
        
        %-- Worst case approximation error is zero        
        Bbd=cell(1,nrNodes);
        for ii=1:nrNodes
            Bbd{ii} = 0.*Bb{ii};
        end
        
        gridpnt = gridpnt0;
        MaxEps = 0;
        MinEps = 0;
        DELTA = 0;
        TRI = [];
        
    case {'line'}
        [Bbd,gridpnt,DELTA,MaxEps,MinEps] = ...
            GNB_line(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,nrNodes,plotflag,verbose);        
        TRI = [];
    case {'area'}
        [Bbd,gridpnt,DELTA,MaxEps,MinEps,TRI] = ...
            GNB_area(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,nrNodes,plotflag,verbose);
end

%% Combine output data

GNBoutput.Bbd     = Bbd;
GNBoutput.gridpnt = gridpnt;
GNBoutput.MaxEps  = MaxEps;
GNBoutput.MinEps  = MinEps;
GNBoutput.TRI     = TRI;
GNBoutput.deltas  = DELTA;

end


% -------------------------------------------------------------------------
% Underlying functions
% -------------------------------------------------------------------------


% ---------------- Case: varying delays -----------------------------------
function [Bbd,gridpnt,DELTA,MaxEps,MinEps] = GNB_line(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,nrNodes,plotflag,verbose)
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%   * constant transmission interval
%   * variable transmission delays
% -------------------------------------------------------------------------
% Created:  04-08-2011 BvanLoon
% Updated:  09-06-2013 NBauer
% -------------------------------------------------------------------------
    
    
    % Gridding and Norm Bounding Procedure
    conv        = 0;
    maxiter     = 2*gridpntM;            % maximum number of allowed iterations
    gridpnt_up  = gridpnt0;              
    iter        = 0;
    
    if plotflag == true
        pic = figure;
        subplot(1,2,1)
        plot(gridpnt0(:,1),gridpnt0(:,2))
        hold on;
        plot(gridpnt0(:,1),gridpnt0(:,2),'bo')
        set(gca,'YTick',[])
        xlabel('transmission intervals (s)');
        ylabel('delays (s)');
        subplot(1,2,2)
        hold on;
        xlabel('Number of iterations')
        ylabel('Maximum value of \epsilon')
    end

    % Define empty cells
    alpha12     = cell(0); 
    DELTA       = cell(0);

    while (~conv)
        
        iter        = iter+1;
        gridpnt     = gridpnt_up;
        nrGridPnts  = size(gridpnt,1);
        
        if plotflag == true
            if iter~=1
                figure(pic)
                subplot(1,2,1);
                plot(new_gridpnt(:,1),new_gridpnt(:,2),'bo')
                set(gca,'YTick',[])
            end
        end  
        
        for ell=1:nrNodes
            
            
            
            if ell <= length(JJ)    %determine if there are node dependent dynamics
                
                nrJdnBlks  = max(size(JJ{ell}));       % total number of Jordan blocks 
                
                if iter == 1
                    nrSegs = nrGridPnts-1;
                else
                    nrSegs = 2;
                end
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %-- Computation of approximation error per segment for each Jordan block
                DELTAup     = zeros(nrSegs,nrJdnBlks);
                alpha12up   = cell(0);
                for m = 1:nrSegs    % for all vertices     

                    % isolate segment(s) that need to be evaluated
                    if iter == 1
                        gridpntSEG = gridpnt(m:m+1,:);         %all segments
                    else
                        M = [rd rd+1 rd+2];
                        gridpntSEG = gridpnt(M(m:m+1),:);   %just new segments
                    end

                    %for segment, setup approximation error norm functions for each Jordan Block
                    deltilde    = genErrorNormFunctions(JJ{ell},gridpntSEG,'line');
                    
                    nrDelTildes = length(deltilde);
                    
                    %determine location and value of max approx error   
                    for ii = 1:nrDelTildes
                        [x FVAL]            = findMaxApproxErr(deltilde{ii},'line');
                        DELTAup(m,ii)       = FVAL;
                        alpha12up{ell,m,ii} = x;
                    end
                end  

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % -- Place DELTAup and alpha12up at the right location
                % ordering must be preserved
                if iter == 1
                    DELTA{ell} = DELTAup;
                    alpha12    = alpha12up;
                else
                    ndel = size(DELTA{ell},1);
                    
                    DELTA{ell} = [DELTA{ell}(1:rd-1,:)
                                  DELTAup
                                  DELTA{ell}(rd+1:end,:)];
                                  
                    alpha12N                            = alpha12(ell,1:rd-1,:);   
                    alpha12N(ell,end+1:end+2,:)         = alpha12up;
                    alpha12N(ell,end+1:end+ndel-rd,:)   = alpha12(ell,rd+1:end,:);
                    
                    alpha12=alpha12N;
                end    

                % Worst case approximation error evaluated for all gridpoints for each real Jordan block
                deltaM = cell(0);
                for q = 1:nrJdnBlks
                    deltaM{ell,q}               = max(DELTA{ell}(:,q))*eye(max(size(JJ{ell}{q})));
                    deltaM{ell,q+nrJdnBlks}     = max(DELTA{ell}(:,q+nrJdnBlks))*eye(max(size(JJ{ell}{q})));
                    deltaM{ell,q+2*nrJdnBlks}   = max(DELTA{ell}(:,q+2*nrJdnBlks))*eye(max(size(JJ{ell}{q})));
                end
            else
                % all nodes have equal dynamics
                deltaM(ell,:)    = deltaM(1,:);
                DELTA(ell)       = DELTA(1);
                alpha12(ell,:,:) = alpha12(1,:,:);
            end    
        end
        
        % Calculate epsilon
        epsilon = zeros(nrNodes,1);
        Bbd     = cell(nrNodes,1);
        for ell = 1:nrNodes
            % Determine Bbd
            Bbd{ell}     = Bb{ell}*blkdiag(deltaM{ell,:});
            epsilon(ell) = norm(Bbd{ell})*norm(Cb{ell});
        end
        MAX_eps = max(epsilon);

        if plotflag == true
            figure(pic)
            subplot(1,2,2)
            semilogy(iter,MAX_eps,'bo-')
            xlabel('Number of iterations')
            ylabel('Maximum value of \epsilon')
            pause(0.5)
        end
        
        if verbose == 1
            disp(['  Iteration ' num2str(iter) ', Gridpoints ' num2str(nrGridPnts) ' ... eps = ' num2str(MAX_eps)])
        end
        
        % Determine if algorithm can be terminated
        conv = (MAX_eps < epsilon_u) || (iter >= maxiter) || (nrGridPnts >= gridpntM);
        if conv == 0    % cannot terminate yet
            % -------------------------
            % NEW GRIDPOINT CALCULATION
            
            %find node which has greatest approx error
            worstNodes = find(epsilon == MAX_eps);
            worstNode  = worstNodes(1);  %may be more than one node
            
            
            [rd,cd] = find(DELTA{worstNode}==max(max(DELTA{worstNode})));
            rd = rd(1);  % if nr of rd>1, take the first one
            cd = cd(1);  % if nr of cd>1, take the first one

            % search for matching alpha
            alphagrid   = alpha12{worstNode,rd,cd};
            new_gridpnt = alphagrid(1,1)*gridpnt(rd,:)+alphagrid(1,2)*gridpnt(rd+1,:);
            
            gridpnt_up  = sortrows([gridpnt;new_gridpnt]);
        else 
            break       % epsilon < epsilon_u  or maximum iterations reached
        end
                
    end % end while

    %algorithm has terminated
    MaxEps = MAX_eps;
    MinEps = min(epsilon);
    
    %-- Information
    if verbose ==1
        disp('  --------------------------------------')
        fprintf('  The maximum value of epsilon = %1.3e \n',MAX_eps)
        disp(['  Total amount of gridpoints   = ' num2str(nrGridPnts)])
    end
end % end function

% ---------------- Case: varying transmission intervals and delays --------
function [Bbd,gridpnt,DELTA,MaxEps,MinEps,TRI] = GNB_area(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,nrNodes,plotflag,verbose)
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%   * variable transmission intervals
%   * variable transmission delays
% -------------------------------------------------------------------------
% Created:  04-08-2011 BvanLoon
% Updated:  09-06-2013 NBauer
% -------------------------------------------------------------------------

    conv        = 0;
    iter        = 0;
    maxiter     = 2*gridpntM;      % maximum number of allowed iterations 
    gridpnt     = gridpnt0;
    TRI         = delaunay(gridpnt0(:,1),gridpnt0(:,2));    %-- Divide Theta into triangles
    
    if plotflag == true
        pic = figure;
        subplot(1,2,1)
        triplot(TRI,gridpnt0(:,1),gridpnt0(:,2),'bo-')
        xlabel('Transmission Intervals (s)')
        ylabel('Delays (s)')
        axis([min(gridpnt0(:,1)*0.9) max(gridpnt0(:,1)*1.1) min(gridpnt0(:,2)*0.9)  max(gridpnt0(:,2)*1.1)])
        hold on
        subplot(1,2,2)
        xlabel('Number of iterations')
        ylabel('Maximum value of epsilon')
        set(gca, 'box','on') 
        hold on
    end

    % Define empty cells
    alpha123   = cell(0); 
    DELTA      = cell(0);

    while (~conv)
        
        iter        = iter+1;
        totalNrTri  = size(TRI,1);
        nrGridPnts  = max(size(gridpnt));
        
        %plot new gridpoint
        if plotflag == true
            figure(pic)
            subplot(1,2,1)
            triplot(TRI,gridpnt(:,1),gridpnt(:,2),'bo-')
            xlabel('transmission intervals (s)')
            ylabel('delay (s)')
            hold on
            pause(0.5)
        end
        
        for ell=1:nrNodes
            
            if ell <= length(JJ)    %determine if there are node dependent dynamics
                
                nrJdnBlks  = max(size(JJ{ell}));       % total number of Jordan blocks 
                
                % isolate trianlge(s) that need to be evaluated
                if iter == 1
                    nrTri = totalNrTri;    % total number of triangles
                else
                    nrTri = nrNewTRI;      % just new triangles
                end
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %-- Computation of approximation error per triangle for each Jordan block to create an initial grid
                DELTAup     = zeros(nrTri,3*nrJdnBlks);
                alpha123up  = cell(0);
                for m = 1:nrTri

                    % isolate triangle
                    if iter==1
                        triangle = TRI(m,:);                  
                    else
                        triangle = TRI((totalNrTri-nrTri+m),:);
                    end

                    gridpntTRI = [gridpnt(triangle(1),:)
                                  gridpnt(triangle(2),:)
                                  gridpnt(triangle(3),:)];  

                    %for triangle, setup approximation error norm functions for each Jordan Block
                    deltilde = genErrorNormFunctions(JJ{ell},gridpntTRI,'area');
                    nrDelTildes = length(deltilde);
                    
                    %determine location and value of max approx error                               
                    for ii = 1:nrDelTildes
                        [x FVAL]              = findMaxApproxErr(deltilde{ii},'area');
                        DELTAup(m,ii)         = FVAL;
                        alpha123up{ell,m,ii}  = x;
                    end
                end  

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % -- Place DELTAup and alpha12up at the right location
                if iter == 1
                    DELTA{ell} = DELTAup;
                    alpha123   = alpha123up;
                else

                   %update DELTA
                   DELTA{ell}(rd,:) = [];                       %delete old value
                   DELTA{ell}       = [DELTA{ell}; DELTAup];    %insert updated value at end

                   %update alpha123
                   alpha123                         = alpha123(ell,[1:rd-1 rd+1:end],:);    %delete old value
                   alpha123(ell,end+1:end+nrTri,:)  = alpha123up;                           %insert new value at end
                end

                %-- Worst case approximation error of ALL triangles for each real Jordan block
                deltaM = cell(0);
                for q = 1:nrJdnBlks
                    deltaM{ell,q}               = max(DELTA{ell}(:,q))*eye(max(size(JJ{ell}{q})));
                    deltaM{ell,q+nrJdnBlks}     = max(DELTA{ell}(:,q+nrJdnBlks))*eye(max(size(JJ{ell}{q})));
                    deltaM{ell,q+2*nrJdnBlks}   = max(DELTA{ell}(:,q+2*nrJdnBlks))*eye(max(size(JJ{ell}{q})));
                end
            else
                % all nodes have equal dynamics
                deltaM(ell,:)       = deltaM(1,:);
                DELTA(ell)      	= DELTA(1);
                alpha123(ell,:,:)   = alpha123(1,:,:);
            end    
        end
        
        
        % Calculate epsilon
        epsilon = zeros(nrNodes,1);
        Bbd     = cell(1,nrNodes);
        for ell = 1:nrNodes
            % Determine Bbd
            Bbd{ell}        = Bb{ell}*blkdiag(deltaM{ell,:});
            epsilon(ell)    = norm(Bbd{ell})*norm(Cb{ell});
        end
        MAX_eps = max(epsilon);
        
        %plot new epsilon max
        if plotflag == true
            figure(pic)
            subplot(1,2,2)
            semilogy(iter,MAX_eps,'bo-')
            xlabel('Number of iterations')
            ylabel('Maximum value of epsilon')
            xlim([0 iter+1])
            pause(0.5)
        end
        
        if verbose ==1 
            disp(['  Iteration ' num2str(iter) ', Gridpoints ' num2str(nrGridPnts) ' ... eps = ' num2str(MAX_eps)])
        end
        
        % Determine if algorithm can be terminated
        conv = (MAX_eps < epsilon_u) | (iter >= maxiter) | (nrGridPnts >= gridpntM);
        if conv == 0    % cannot terminate yet
            % NEW GRIDPOINT CALCULATION

            %find segment which has greatest approx error
            worstNodes = find(epsilon == MAX_eps);
            worstNode  = worstNodes(1);  %may be more than one node
            
            [rd,cd] = find(DELTA{worstNode}==max(max(DELTA{worstNode}))); 
            
            if length(rd)>1
                rd = rd(1); % if nr of rd>1, take the first one
                cd = cd(1); % if nr of cd>1, take the first one
            end
            
            %-- Search for matching alpha
            alphagrid = alpha123{ell,rd,cd};
            alphagrid(alphagrid<=eps) = 0;  % set to 'real' zero to prevent num. probl.

            %- Search for matching triangle and gridpoints
            triangleLE   = TRI(rd,:);                    % isolate triangle
            gridpntTRILE = [gridpnt(triangleLE(1),:)
                            gridpnt(triangleLE(2),:)
                            gridpnt(triangleLE(3),:)];
            
            %indicate triangle which will be divided
            if plotflag == true
                figure(pic)
                subplot(1,2,1)
                %highlight triangle
                patch(gridpntTRILE(:,1),gridpntTRILE(:,2),'r')
                hold on
                pause(0.5)
                figure(pic)
                %remove highlight
                subplot(1,2,1)
                patch(gridpntTRILE(:,1),gridpntTRILE(:,2),'w')
                hold on
            end

            %-- Calculate location new gridpoint
            %   Round to six decimals to prevent equal grid points due to numerical
            %   inaccuracy
            newgridpnt = round((alphagrid*gridpntTRILE)*1e6)/1e6;
            %-- Subdivide triangle with the largest error into new triangles
            [newgridpnts,TRI_new,nrNewTRI] = triangulationGND(TRI,gridpnt,alphagrid,newgridpnt,rd);
            
            %update gridpnt and TRI
            gridpnt     = [gridpnt; newgridpnts];   %#ok<AGROW>
            TRI(rd,:)   = [];                       %delete triangle with largest approx error
            TRI         = [TRI;TRI_new];            %#ok<AGROW> %add new sub triangles
        else 
            break       % epsilon < epsilon_u  or maximum iterations reached
        end
        
    end

    %algorithm has terminated
    MaxEps = MAX_eps;
    MinEps = min(epsilon);

    %-- Information
    if verbose ==1
        disp('  --------------------------------------')
        fprintf('  The maximum value of epsilon = %1.3e \n',MAX_eps)
        disp(['  Total amount of gridpoints   = ' num2str(nrGridPnts)])
    end
end

% Function to create detilde function handles which will be solved by fimincon
function deltilde = genErrorNormFunctions(JJ,gridpnt,type)
    %-- Determiniation of the approximation error functions
    %   There are two cases:
    %       * 'RD'   - real and distinct eigenvalues;
    %       * 'RnDC' - real and nondistinct or complex eigenvalues
    %         (longer computational time due to the matrix exponential)
    
    nrJdnBlks  = max(size(JJ));       % total number of Jordan blocks
    
    % Search for the maximum size of the Jordan Blocks
    Msize = zeros(nrJdnBlks,1);
    for q = 1:nrJdnBlks
        Msize(q,1) = max(size(JJ{q}));
    end

    if max(Msize)== 1
        jdnType = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
    else
        jdnType = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
    end   
    
    switch type
        case 'area'
            
            fun1 = @(alpha) alpha(1)*gridpnt(1,1)+alpha(2)*gridpnt(2,1)+alpha(3)*gridpnt(3,1);
            fun2 = @(alpha) alpha(1)*(gridpnt(1,1)-gridpnt(1,2))+alpha(2)*(gridpnt(2,1)-gridpnt(2,2))+alpha(3)*(gridpnt(3,1)-gridpnt(3,2));          
            
            deltilde = cell(3*nrJdnBlks,1);
            switch jdnType
                case 'RD'
                    for q = 1:nrJdnBlks
                        if abs(JJ{q}) > 1e-14

                            deltilde{q} =   @(alpha) -abs(exp(JJ{q}*(fun1(alpha)))...
                                                         -alpha(1)*exp(JJ{q}*gridpnt(1,1))...
                                                         -alpha(2)*exp(JJ{q}*gridpnt(2,1))...
                                                         -alpha(3)*exp(JJ{q}*gridpnt(3,1)));     

                            deltilde{q+nrJdnBlks} = @(alpha) -abs((alpha(1)/JJ{q})*(exp(JJ{q}*(fun1(alpha)))-exp(JJ{q}*gridpnt(1,1)))...
                                                                 +(alpha(2)/JJ{q})*(exp(JJ{q}*(fun1(alpha)))-exp(JJ{q}*gridpnt(2,1)))...
                                                                 +(alpha(3)/JJ{q})*(exp(JJ{q}*(fun1(alpha)))-exp(JJ{q}*gridpnt(3,1))));

                            deltilde{q+2*nrJdnBlks} = @(alpha) -abs((alpha(1)/JJ{q})*(exp(JJ{q}*(fun2(alpha)))-exp(JJ{q}*(gridpnt(1,1)-gridpnt(1,2))))...
                                                                   +(alpha(2)/JJ{q})*(exp(JJ{q}*(fun2(alpha)))-exp(JJ{q}*(gridpnt(2,1)-gridpnt(2,2))))...
                                                                   +(alpha(3)/JJ{q})*(exp(JJ{q}*(fun2(alpha)))-exp(JJ{q}*(gridpnt(3,1)-gridpnt(3,2)))));

                        else
                            deltilde{q}     = @(alpha) 0;
                            deltilde{q+nrJdnBlks}   = @(alpha) 0;
                            deltilde{q+2*nrJdnBlks} = @(alpha) 0;
                        end
                    end
                case 'RnDC'
                    syms s
                    for q = 1:nrJdnBlks

                        deltilde{q} = @(alpha) -norm(double(expm(JJ{q}*(fun1(alpha))))...
                                                    -double(alpha(1)*expm(JJ{q}*gridpnt(1,1)))...
                                                    -double(alpha(2)*expm(JJ{q}*gridpnt(2,1)))...
                                                    -double(alpha(3)*expm(JJ{q}*gridpnt(3,1))));

                        deltilde{q+nrJdnBlks} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{q}*s),s,gridpnt(1,1),(fun1(alpha))))...
                                                              +double(alpha(2)*int(expm(JJ{q}*s),s,gridpnt(2,1),(fun1(alpha))))...
                                                              +double(alpha(3)*int(expm(JJ{q}*s),s,gridpnt(3,1),(fun1(alpha)))));

                        deltilde{q+2*nrJdnBlks} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{q}*s),s,(gridpnt(1,1)-gridpnt(1,2)),(fun2(alpha))))...
                                                                +double(alpha(2)*int(expm(JJ{q}*s),s,(gridpnt(2,1)-gridpnt(2,2)),(fun2(alpha))))...
                                                                +double(alpha(3)*int(expm(JJ{q}*s),s,(gridpnt(3,1)-gridpnt(3,2)),(fun2(alpha)))));    
                    end
            end
        case 'line'
            
            fun1 = @(alpha) alpha(1)*gridpnt(1,1)+alpha(2)*gridpnt(2,1);
            fun2 = @(alpha) alpha(1)*(gridpnt(1,1)-gridpnt(1,2))+alpha(2)*(gridpnt(2,1)-gridpnt(2,2));          
            
            deltilde = cell(3*nrJdnBlks,1);
            switch jdnType
                case 'RD'
                    for q = 1:nrJdnBlks
                        if abs(JJ{q}) > 1e-14

                            deltilde{q} =   @(alpha) -abs(exp(JJ{q}*(fun1(alpha)))...
                                                         -alpha(1)*exp(JJ{q}*gridpnt(1,1))...
                                                         -alpha(2)*exp(JJ{q}*gridpnt(2,1)));     

                            deltilde{q+nrJdnBlks} = @(alpha) -abs((alpha(1)/JJ{q})*(exp(JJ{q}*(fun1(alpha)))-exp(JJ{q}*gridpnt(1,1)))...
                                                                 +(alpha(2)/JJ{q})*(exp(JJ{q}*(fun1(alpha)))-exp(JJ{q}*gridpnt(2,1))));

                            deltilde{q+2*nrJdnBlks} = @(alpha) -abs((alpha(1)/JJ{q})*(exp(JJ{q}*(fun2(alpha)))-exp(JJ{q}*(gridpnt(1,1)-gridpnt(1,2))))...
                                                                   +(alpha(2)/JJ{q})*(exp(JJ{q}*(fun2(alpha)))-exp(JJ{q}*(gridpnt(2,1)-gridpnt(2,2)))));

                        else
                            deltilde{q}     = @(alpha) 0;
                            deltilde{q+nrJdnBlks}   = @(alpha) 0;
                            deltilde{q+2*nrJdnBlks} = @(alpha) 0;
                        end
                    end
                case 'RnDC'
                    syms s
                    for q = 1:nrJdnBlks

                        deltilde{q} = @(alpha) -norm(double(expm(JJ{q}*(fun1(alpha))))...
                                                    -double(alpha(1)*expm(JJ{q}*gridpnt(1,1)))...
                                                    -double(alpha(2)*expm(JJ{q}*gridpnt(2,1))));

                        deltilde{q+nrJdnBlks} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{q}*s),s,gridpnt(1,1),(fun1(alpha))))...
                                                              +double(alpha(2)*int(expm(JJ{q}*s),s,gridpnt(2,1),(fun1(alpha)))));

                        deltilde{q+2*nrJdnBlks} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{q}*s),s,(gridpnt(1,1)-gridpnt(1,2)),(fun2(alpha))))...
                                                                +double(alpha(2)*int(expm(JJ{q}*s),s,(gridpnt(2,1)-gridpnt(2,2)),(fun2(alpha)))));    
                    end
            end  
    end

end

% Function to find maximum of detilde function handle using fmincon
function [convComb FVAL] = findMaxApproxErr(deltilde,type)

    %-- Parameters fmincon
    switch type
        case 'line'
            X0 = [0.5 0.5];
            Afc = [];     bfc = [];
            Aeq = [1 1];  beq = 1;
            lb = [0;0];   ub = [1;1];
            nonlon = [];
            options = optimset('algorithm','sqp','display','off','TolFun',1e-10,'TolX',1e-12,'LargeScale','off');
        case 'area'
            %-- Parameters fmincon
            X0 = [1/3 1/3 1/3];
            Afc = [];       bfc = [];
            Aeq = [1 1 1];  beq = 1;
            lb = [0;0;0];   ub = [1;1;1];
            nonlon = [];
            options = optimset('algorithm','sqp','display','off','TolFun',1e-10,'TolX',1e-12,'LargeScale','off');
    end
    
    [convComb FVAL] = fmincon(deltilde,X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options);
    FVAL=-FVAL;
    %FVAL = -deltilde(convComb);
end

% Function to subdivide triangles into new triangles
function [newgridpnt,TRI_new,nrNewTriRI_up] = triangulationGND(TRI,gridpnt,alphagrid,newgridpnt,rd)
% this function isolates the triangle with the largest approximation error 
% and divides this triangle into new triangles

    Triangle = TRI(rd,:);
    nrGridPnts = length(gridpnt);
        
    % Search for equal grid points
    eqG = zeros(max(size(gridpnt)),1);
    for ii=1:max(size(gridpnt))
        eqG(ii,1) = isequal(gridpnt(ii,:),newgridpnt);
    end

    eqN = find(eqG==1);

    if isempty(eqN)==1
        % no equal gridpoints
        new = nrGridPnts+1;
    else
        % equal gridpoint found
        new = eqN;
    end

    % subdivide triangles, new point is within interior
    if sum(alphagrid==0)== 0
        q1 = Triangle(1);
        q2 = Triangle(2);
        q3 = Triangle(3);
        TRI_new = [new q1 q2
                   new q1 q3
                   new q2 q3];
        nrNewTriRI_up = 3;
    % subdivide triangles, new point is on a boundry line
    elseif sum(alphagrid==0)==1
        hp = find(alphagrid>0);     % find gridpoints where alpha_j ~= zero
        hz = find(alphagrid==0);    % find gridpoint where alpha_j == zero
        TRI_new = [new Triangle(hz) Triangle(hp(1))
                   new Triangle(hz) Triangle(hp(2))];
        nrNewTriRI_up = 2;
    % subdivide triangles, new point is on another existing gridpoint
    else
        error('GNB ERROR: Attempted to add gridpoint on top of another gridpoint.')
    end
    
%     if isempty(eqN)==1
%         % no equal gridpoints
%         gridpnt_up = [gridpnt ; newgridpnt];
%     else
%         % equal gridpoint found
%         gridpnt_up = gridpnt;
%     end

end


