function [S alfa] = jnf(J)
%JNF   Determines matrices S{i} and symbolic terms alfa{i} such that the  
%      matrix exponential expm(Jb*hk) can be expressed in Jordan Normal Form 
%      as sum(alfa{i}*S{i}).
% 
%   [S ALFA] = JNF(J) determines the cells S and ALFA such that 
%   expm(Jb*hk) = sum(ALFA{i}*S{i}), where Jb is a matrix in Jordan normal form,
%   and hk is a symbolic variable. The input J is a cell containing the individual   
%   Jordan blocks of Jb. The output of this function is a cell S containing   
%   constant matrices and a cell of symbolic variables ALFA as functions of hk 
%   such that expm(Jb*hk) = sum(ALFA{i}*S{i}). 
%
%   See also JNF2CH, RJD, SEPJDNBLOCKS
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    syms hk
    sizeJ = size(J,1);
    alfa = cell(1,sizeJ);       
    S = cell(1,sizeJ); 

    if size(J,1) > 1
        JLdiag = J(2,1);
    else
        JLdiag =0;
    end

    if JLdiag == 0  %eigenvalues are real

        for i = 1:sizeJ
            alfa{i} = hk^(i-1)*exp(J(1,1)*hk);
            S{i} = 1/factorial(i-1)*(triu(ones(sizeJ),i-1)-triu(ones(sizeJ),i));
        end

    else            %eigenvalues are complex
        sigma = J(1,1);
        omega = J(2,1);

        Ssinus = [0 -1; 1 0];
        for j = 1:(sizeJ/2-1)
            Ssinus = blkdiag(Ssinus,[0 -1; 1 0]);
        end

        for i = 1:sizeJ/2
            if i == 1
                alfa{i} = exp(sigma*hk)*cos(omega*hk);
                alfa{i+sizeJ/2} = exp(sigma*hk)*sin(omega*hk);
            else
                alfa{i} = hk^(i-1)*exp(simga*hk)*cos(omega*hk);
                alfa{i+sizeJ/2} = hk^(i-1)*exp(sigma*hk)*sin(omega*hk);
            end
            S{i} = 1/factorial(i-1)*(triu(ones(sizeJ),2*i-2)-triu(ones(sizeJ),2*i-1));
            S{i+sizeJ/2} = S{i}*Ssinus;
        end
    end
    
end
