function [Ai f] = jnf2ch(A,S,alfa)
%JNF2CH    Determines the matrices Ai{i} and symbolic terms f{i} such that 
%          the matrix exponential expm(A*hk) can be expressed as sum(f{i}*Ai{i})
%          by using the Cayley-Hamilton theorem and the output from jnf.m. See 
%          [Gielen et. al., Automatica 2010] for more information.
%
%   [AI F] = JNF2CH(A,S,ALFA) determines the matrices AI{i} and symbolic  
%   terms F{i} by using the Cayley-Hamilton theorem to express the 
%   matrix exponential e^(A*hk) = sum(F{i}*AI{i}). It does so by using S{i}  
%   and ALFA{i} which result from the JNF overapproximation procedure, i.e.
%   e^(A*hk) = sum(ALFA{i}*S{i}), and the A matrix of the system. 
%
%   See also JNF
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% compute new S terms for Cayley-Hamilton

    Ai = cell(0);                            
    for i = 1:size(S,2)               
        Ai{i} = A^(i-1);
    end

    matA = zeros(size(A,1)*size(A,1),size(Ai,2));   
    for j = 1:size(Ai,2)
        matA(:,j) = vec(Ai{j});                         %create a matrix whose columns are columns of A^{j-1}
    end

    % solve for vector beta in problem matJ*beta = vec(S{i})
    beta = cell(1,size(Ai,2));                     
    for i = 1:size(Ai,2)               
        beta{i} =matA\(vec(S{i}));
    end


    f = cell(1,size(Ai,2));                        
    for i = 1:size(Ai,2)               
        f{i} = 0;
        for j = 1:size(Ai,2)
            f{i} = f{i} + beta{j}(i)*alfa{j};          % f  = sum beta{j}(i)*alfa{j}
        end
    end
    
end