
function JJ = sepJdnBlocks(J)
%SEPJDNBLOCKS   Seperates a matrix that is in real Jordan form into 
%               individual jordan blocks.
%
%   JJ = SEPJDNBLOCKS(J) Takes a matrix J, which is in real jordan form,
%   and outputs a cell JJ, containing the individual Jordan blocks.
%
%   See also RJD
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    % Initialize output
    JJ = cell(0);

    % Check for trivial cases first
    switch size(J,1)
    case 0, JJ = {};      % Empty
    case 1, JJ = { J };  % Case 1x1
    case 2,               % Case 2x2
      if (J(1,2) ~= 0)
        JJ = {J};
      else
        JJ = {J(1,1), J(2,2)};
      end;

    otherwise             % Case >= 3x3              

      % Extract diagonal and superdiagonal into vector data (with padding)
      data = zeros(size(J,1),4);
      data(1:end-1,1) = diag(J,-1);
      data(1:end  ,2) = diag(J, 0);
      data(1:end-1,3) = diag(J, 1);
      data(1:end-2,4) = diag(J, 2);

      % ... walk over the data rows (i.e. diagonal entries)
      while size(J,1) > 0;

        % Check whether entry is complex (otherwise it is real)
        %   -- set blocksize and multiplicity diagonal
        if (abs(data(1,1)+data(1,3))<1e-12 && abs(data(1,1))> 1e-12)
          bs = 2; md = 4;
        else
          bs = 1; md = 3;
        end;

        % Check multiplicity
        k = bs;
        while (data(k, md) == 1)
          k = k + bs;
        end;
        JJ(length(JJ)+1) = {J(1:k,1:k)};

        % Remove processed part (shrink data and J matrices) of the input matrix (eat it)
        data = data(k+1:end,:);
        J = J(k+1:end,k+1:end);

      end;
    end;

end

