function [Hmati,Tmad] = analyzeHybNcs(hybMdl,l,protocol)
% analyzeHybNcs This function computes the maximum allowable transmission
%               interval (MATI), maximum allowable delay (MAD) tradeoff   
%               curve for a Networked Control System according to the procedure
%               in [Heemels et. al., Trans. Autom. Control 2010]. 
%
%   [MATI,MAD] = analyzeHybNcs(HYBNCSMDL,L,PROTOCOL)  Generates the vectors MATI, MAD  
%   needed to plot a maximum allowable transmission interval (MATI), maxmium allowable
%   delay (MAD) tradeoff curve which defines a region where robust stability can 
%   be guaranteed for a HYBNCSMDL. 
%
%   INPUTS
%   ------------------------------------------------------
%      HYBNCSMDL   - structure containing matrices which describe the NCS
%                    as a hybrid model.  All fields specified in the 
%                    output of genHybNcsMdl.m are required fields of HYBNCSMDL.
%      L           - Total number of nodes
%      PROTOCOL    - 'RR' or 'TOD'
%
%   OUTPUTS
%   ------------------------------------------------------
%      MATI        - vector of x values for MATI,MAD tradeoff plot
%      MAD         - vector of y values for MATI,MAD tradeoff plot
%
%   Customization Tips:
%      *The structure which is input, HYBNCSMODEL, can be generated by a 
%       custom hybrid NCS model of the form detailed in
%       [Heemels et. al., Trans. Autom. Control 2010].
%
%   See also GENHYBNCSMODEL
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    disp(' ')
    disp('STABILITY DATA:')
    disp('   Generating MATI MAD tradeoff curve ... ')
%% Pick Stability Analysis

    [Hmati,Tmad] = checkStability_diffEqCompare_RR_TOD(hybMdl,l,protocol);

end

function [Hmati,Tmad] = checkStability_diffEqCompare_RR_TOD(hybMdl,l,protocol)

    A11 = hybMdl.A11;
    A12 = hybMdl.A12;
    A21 = hybMdl.A21;
    A22 = hybMdl.A22;

    % ==========
    % program calculating constants (L_0, L_1, gamma_0, gamma_1)
    % ==========

    % Calculate constants depending of protocol according to [Lemma V.4]
    if strcmp(protocol,'RR')
        lambda = sqrt((l-1)/l);
        lambdaW = sqrt(l);
        alpha_low = 1;
        %alpha_high = sqrt(l);
        M1 = sqrt(l);
    elseif strcmp(protocol,'TOD')
        lambda = sqrt((l-1)/l);
        lambdaW = 1;
        alpha_low = 1;
        %alpha_high = 1;
        M1 = 1;
    else
        error('protocol should be either ''RR''or ''TOD''')
    end



    % ==========
    % program computing L2 gain from e to m_x gamma by solving LMIs [(57)]
    % ==========
    
    % setup matrices and sdpvars
    eps = 1e-6;
    [m,n] = size(A21);
    P = sdpvar(n,n);
    gamma_s = sdpvar(1,1);
    % LMI definitions
    L1 = set([A11.'*P+P*A11+eps*eye(n)+A21.'*A21 P*A12; A12.'*P (eps-gamma_s)*eye(m)]<0);
    L2 = set(P>0);
    L  = L1+L2;
    % Calls SeDuMi solver to solve LMI
    %disp(strvcat('Calling SeDuMi solver to solve LMI',' '))
    solvesdp(L,gamma_s,sdpsettings('verbose',0));
    % Recover solution from SDP variables
    P = double(P);
    gamma_s = double(gamma_s);
    gamma = sqrt(gamma_s);
    % Testing the LMI outcome!
    test_LMI = round(eig([A11'*P+P*A11+eps*eye(n)+A21'*A21 P*A12; A12'*P (eps-gamma_s)*eye(m)])*10^6)/10^6;
    test_P   = round(eig(P)*10^6)/10^6;
    if any(test_P < 0) || any(test_LMI > 0)
        Hmati=[];
        Tmad=[];    
        disp('   Cannot compute L2 gain from e to m_x, aborting...')
        return;
    end

    % ==========
    % program computing constants for differential equations
    % split into two parts, depending of sampled data (l==1) and NCS (l>1)
    % ==========

    % Computing Me
    Me = sqrt(max(abs(eig(A22.'*A22))));

    % Computing constants for differential equations phi [1, (22)] from [1, Theorem V.3]
    if strcmp(protocol,'RR')
        % V 0.2: Sharper bound see [1]
        L_0 = (Me)/(alpha_low);
    elseif strcmp(protocol,'TOD')
        L_0 = (M1*Me)/(alpha_low);
    else
        error('protocol should be either ''RR''or ''TOD''')
    end

    if l>1
        %disp(strvcat(' ','number of nodes, l > 1'))
        L_1 = L_0*(lambdaW/lambda);
        gamma_0 = M1*gamma;
        gamma_1 = (M1*gamma*lambdaW)/lambda;

        % ==========
        % program solving phi functions by ode45 solver for different starting
        % conditions
        % ==========
        %disp(strvcat(' ','Solving differential equations Phi(t) for different initial conditions'))
        % initial conditions on differential equations phi
        % phi10 = phi00, so starting with delay free case.
        % in the loop phi01 is made higher then phi 00, so delayed cases are
        % calculated
        phi00 = 1/lambda;
        phi10 = phi00;
        options = odeset('Maxstep',1e-4,'Events',@(t,phi) events(phi,phi10,lambda));

        % initialize variables; tmati arbitratry but tmati>0, tmad = 0, since
        % phi10 = phi01, delay free case [1]
        T = [];
        tmad = .1; tmati = 100;
        a = .0001;
        % iterating for higher delays, finding tmati-tmad combinations in a grid of
        % initial conditions
        while tmad <= tmati
            % saving previous tmad
            tmadp = tmad;
            % solving
            [t,phi] = ode45(@(t,phi) Phi(phi,L_0,L_1,gamma_0,gamma_1),[0 100],[phi00,phi10],options);
            % finding solution for Hmati according to [Theorem IV.2]
            tmati = max((phi(:,1) >= lambda^2*phi10).*t);
            % finding solution for Tmad according to [Theorem IV.2]
            tmad = max((phi(:,2) >= phi(:,1)).*t);
            if tmad <= tmati
                % save new combination of Hmati-Tmad in T
                T = [T; [tmati tmad]]; %#ok<AGROW>
            end
            % increase or decrease change in init condition, according to
            % relative difference in tmad
            if (tmad - tmadp)/(tmadp+.01) > 0.05
                % iterate more (increase in tmad above 5%)
                a = a/1.2;
            elseif (tmad - tmadp)/(tmadp+.01) < 0.01
                % iterate less (increase in tmad below 1%)
                a = a*1.2;
            end
            % set gridded beginning conditions for next iteration
            phi10 = phi10 + a;
        end
    elseif l == 1
            %disp(strvcat(' ','Number of nodes, l, is equal to 1 so sampled data case'))
            %disp('Making iterations for different values for lambda')
        lambda = .1;
        T1 = [];
        while lambda < sqrt((2-1)/2)
            L_1 = L_0*(lambdaW/lambda);
            gamma_0 = M1*gamma;
            gamma_1 = (M1*gamma*lambdaW)/lambda;

            % ==========
            % program solving phi functions by ode45 solver for different starting
            % conditions
            % ==========
            % disp(strvcat(' ','Solving differential equations Phi(t) for different initial conditions'))
            % disp(strvcat('lambda = ',num2str(lambda)))
            % initial conditions on differential equations phi
            % phi10 = phi00, so starting with delay free case.
            % in the loop phi01 is made higher then phi 00, so delayed cases are
            % calculated
            phi00 = 1/lambda;
            phi10 = phi00;
            options = odeset('Maxstep',1e-4,'Events',@(t,phi) events(phi,phi10,lambda));

            % initialize variables; tmati arbitratry but tmati>0, tmad = 0, since
            % phi10 = phi01, delay free case

            tmad = .1; tmati = 100;
            a = .0001;
            % iterating for higher delays, finding tmati-tmad combinations in a grid of
            % initial conditions
            while tmad <= tmati
                % saving previous tmad
                tmadp = tmad;
                % solving
                [t,phi] = ode45(@(t,phi) Phi(phi,L_0,L_1,gamma_0,gamma_1),[0 100],[phi00,phi10],options);
                % finding solution for Hmati according to [Theorem IV.2]
                tmati = max((phi(:,1) >= lambda^2*phi10).*t);
                % finding solution for Tmad according to [Theorem IV.2]
                tmad = max((phi(:,2) >= phi(:,1)).*t);
                if tmad <= tmati
                    % save new combination of Hmati-Tmad in T
                    T1 = [T1; [tmati tmad]]; %#ok<AGROW>
                end
                % increase or decrease change in init condition, according to
                % relative difference in tmad
                if (tmad - tmadp)/(tmadp+.01) > 0.05
                    % iterate more (increase in tmad above 5%)
                    a = a/1.2;
                elseif (tmad - tmadp)/(tmadp+.01) < 0.01
                    % iterate less (increase in tmad below 1%)
                    a = a*1.2;
                end
                % set gridded beginning conditions for next iteration
                phi10 = phi10 + a;
            end
            lambda = lambda*1.25;
        end
        % select least conservative parts of the curves
        T2 = flipdim(sortrows(T1,1),1);
        tmad = T2(1,2);
        T = [];
        for q = 1:length(T2)
            if T2(q,2) > tmad
                T = [T;T2(q,:)]; %#ok<AGROW>
                tmad = T2(q,2);
            else
                % discard
            end
        end
    else
        error('There should be at least one node (sampled data)')
    end

    % ==========
    % output
    % ==========

    % Hmati-Tmad tradeoff curve
    Hmati = T(:,1); Tmad = T(:,2);
    disp('   MATI MAD tradeoff curve complete!')

end

function dphi = Phi(phi,L_0,L_1,gamma_0,gamma_1)%,L_0,L_1,gamma_0,gamma_1)
% ====================
% Authors: P.J.H. Maas, W.P.M.H. Heemels
% Date: Februari 22th, 2010
% Project: Sum of Squares methods for Stability Analysis of Networked Control Systems, 
% Master Thesis
% 
% Version 1.0
% 
% File description:
% This function describes differential equations /dot{phi}_{0} and
% /dot{phi}_{1} from [1] for integration by an ode solver.
% BE CAREFUL WITH GLOBAL VARIABLES!
% ====================

% global L_0 L_1 gamma_0 gamma_1 (now in function call)

    phi0 = phi(1);
    phi1 = phi(2);

    dphi = [( -2*L_0*phi0 - gamma_0*(phi0^2 + 1) );
            ( -2*L_1*phi1 - gamma_0*(phi1^2 + gamma_1^2/gamma_0^2) )];

end

function [value,isterminal,direction] = events(phi,phi10,lambda)%,phi01)
% Locate the time when height passes through zero in a 
% decreasing direction and stop integration.
% global phi10 lambda (now in function call)
  
    if (phi(:,1) < lambda^2*phi10);  % Detect (phi(:,1) < lambda^2*phi10)
        value = 1;  % do detect and stop
    else
        value = 0;  % do not detect and do not stop
    end
    isterminal = 1;  % Stop the integration
    direction = 0;   % Direction does not matter

end

